#include "CAalgos.h"



/* Simple channel assignment where all the nodes
   have one radio assigned the same channel
*/
double OneChannel::Run (Topology *T)
{
//	Topology* T = new Topology(*Gi);     /* copy of the initial graph */
	T->ResetChannels();

	//for each node, one of the radios is assigned channel 1
	for (int u = 0; u < T->GetNumNodes(); u++)
		SetChannel (T->Node(u)->Info()->Radio(0), 1); 

	// each node can reach its neighbors through that radio
	// (this implies that the channel of each link is set to 1)
	for (int u = 0; u < T->GetNumNodes(); u++)
		for (TGraphNeighborIterator v = T->Node(u)->NeighborBegin(); v != T->Node(u)->NeighborEnd(); v++)
			SetRadio (v, &(*T->Node(u)->Info()->Radio(0)));

	// compute Utot for each link (initially the collision domain equals the neighborhood)
	// for every directed link u->v
	for (int u = 0; u < T->GetNumNodes(); u++)
		for (TGraphNeighborIterator v = T->Node(u)->NeighborBegin(); v != T->Node(u)->NeighborEnd(); v++)
			SetUtot (v, T->computeUtot(T->LinkItrs(T->Node(u),v)));

	return 0;
}



CAalgo* CAalgo::Create (string algo, const list<pair<string,string> >& params)
{
  CAalgo* ca;

  if (algo == "DoNothing")
    ca = new CAalgo;
  else if (algo == "OneChannel")
    ca = new OneChannel;
  else if (algo == "IATC")
    ca = new IATC;
  else if (algo == "LACA")
    ca = new LACA;
  else if (algo == "FCPRA")
    ca = new FCPRA;
  else if (algo == "MVCRA")
    ca = new MVCRA;
  else if (algo == "MPCARA")
    ca = new MPCARA;
  else if (algo == "MPCARA_OPT")
    ca = new MPCARA_OPT;
  else if (algo == "E2CARA-TD")
    ca = new E2CARA_TD;
  else if (algo == "PACA")
    ca = new PACA;
  else if (algo == "E2CAR_OPT")
    ca = new E2CAR_OPT;
  else
    return NULL;

  ca->Configure (params);
  ca->SetName ();

  return ca;
}

