#include "CAalgos.h"

#include <iostream>
#include <algorithm>
#include <cassert>

struct PEcard
{
	bool operator()(const set<TLinkItrs> &x, const set<TLinkItrs> &y)
	{
		return x.size() > y.size();
	}
};


/* Link l has been extracted and no common channels have been found
  Replaces channel k1 with k on a radio of the end node j */
void LACA::replace(Topology *T, TLinkItrs l, int j, unsigned int k, unsigned int k1)
{
	/* Find the radio of j which is assigned channel k1 */
// 	RadioIface *r;
// 	for (r=T->node[j].info->radio; r->channel!=k1; r++);
	/* replace k1 with k */
//	r->channel = k;
// 	for (vector<RadioIface>::iterator r=T->node[j].info->radio.begin(); r!=T->node[j].info->radio.end(); r++)
// 	  if (r->channel == k1)
// 	  {
// 	    r->channel = k;
// 	    break;
// 	  }
	int rk = T->Node(j)->Info()->findRadioWithChannel (k1);
	SetChannel (T->Node(j)->Info()->Radio(rk), k);

	/* assign k to the extracted link l */
	T->AssignChannel2Link(l, k );

	for (int u=0; u < T->GetNumNodes(); u++)
		for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++)
			/* if u->v is a link leaving or entering node j and has been assigned channel k */
			if ( (u == j || v->Id() == j) && (v->Radio()!=NULL) && (v->RadioChannel() == k) )
			{
				set<unsigned int> uChannels, vChannels, CommonChannels;

// 				findChannels(u,uChannels);
// 				findChannels(v,vChannels);
				uChannels = T->Node(u)->Info()->findChannels();
				vChannels = v->Info()->findChannels();
				CommonChannels.clear();

				/* compute the intersection between uChannels and vChannels */
				set_intersection(uChannels.begin(), uChannels.end(), vChannels.begin(), vChannels.end(), inserter(CommonChannels, CommonChannels.begin()));
	
				if (!CommonChannels.empty())
					T->AssignChannel2Link(T->LinkItrs(T->Node(u),v), *(CommonChannels.begin()) );
				else
				{
					int w = u == j ? v->Id() : u;
					replace(T,T->LinkItrs(T->Node(u),v),w,k,k1);
				}
			}
}



/* Given the initial graph, performs a channel assignment
   and returns the induced graph
*/
double LACA::Run (Topology *T)
{
//	Topology* T = new Topology(*Gi);     /* copy of the initial graph */

	/* for each link find the set of potentially interfering links */
	multimap<set<TLinkItrs>, TLinkItrs, PEcard> PE;		/* the map is sorted in descending order of |PE[e]| */

	for (int u=0; u<T->GetNumNodes(); u++)
		for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++)
			{
				TLinkItrs e(T->LinkItrs(T->Node(u),v));
				set<TLinkItrs> PEe;
/* if i<->j potentially interfere with e, PEe only contains the pair <i,j> such that i->info->id > j->info->id */

				for (int i=0; i<T->GetNumNodes(); i++)
					for (TGraphNeighborIterator j=T->Node(i)->NeighborBegin(); j!=T->Node(i)->NeighborEnd(); j++)
						if (T->LinkItrs(T->Node(i),j).PotInterferesWith(T->LinkItrs(T->Node(u),v)))
							PEe.insert(T->LinkItrs(T->Node(i),j));

				PE.insert(pair<set<TLinkItrs>, TLinkItrs>(PEe, e));
			}

	for (multimap<set<TLinkItrs>, TLinkItrs, PEcard>::const_iterator e=PE.begin(); e!=PE.end(); ++e)
	{
		set<unsigned int> uChannels, vChannels, CommonChannels;
		/* end nodes of e */
		TGraphNodeIterator u=e->second.first;
		TGraphNeighborIterator v=e->second.second;

// 		findChannels(u,uChannels);
// 		findChannels(v,vChannels);
		uChannels = u->Info()->findChannels();
		vChannels = v->Info()->findChannels();
		CommonChannels.clear();

		/* compute the intersection between uChannels and vChannels */
		set_intersection(uChannels.begin(), uChannels.end(), vChannels.begin(), vChannels.end(), inserter(CommonChannels, CommonChannels.begin()));

		if (!CommonChannels.empty())
			T->AssignChannel2Link(e->second, *(CommonChannels.begin()) );
		else if ( (CommonChannels.empty()) && (uChannels.size()<u->Info()->numAvailRadios()) && (vChannels.size()<v->Info()->numAvailRadios()) )
		{
			vector<double> load(WMS.numAvailChannels()+1);
			for (vector<double>::iterator i=load.begin(); i!=load.end(); i++) *i=0.;

			/* find the load of each channel in PE[e] */
			for (set<TLinkItrs>::iterator l=e->first.begin(); l!=e->first.end(); l++)
				if (l->second->Radio() != NULL)
					load[l->second->RadioChannel()]+=l->second->Flow();

			/* find the least used channel in PE[e] */
			unsigned int k = min_element(++load.begin(), load.end()) - load.begin();
			assert(k>0);

			/* assign k to the link */
			T->AssignChannel2Link(e->second, k );
		}
		else if ( (CommonChannels.empty()) && (uChannels.size()<u->Info()->numAvailRadios()) && (vChannels.size()==v->Info()->numAvailRadios()) )
		{
			vector<double> load(vChannels.size());
			for (vector<double>::iterator i=load.begin(); i!=load.end(); i++) *i=0.;

			/* find the load of the channels of v in PE[e] */
			set<unsigned int>::iterator pos;
			int j;
			for (set<TLinkItrs>::iterator l=e->first.begin(); l!=e->first.end(); l++)
				if ( (l->second->Radio() != NULL) && ((pos=vChannels.find(l->second->RadioChannel())) != vChannels.end()) )
				{
					j=0;
					for (set<unsigned int>::iterator temp=vChannels.begin(); temp!=pos; temp++)
						j++;
					load[j]+=l->second->Flow();
				}

			/* find the least used channel in PE[e] */
			j = min_element(load.begin(), load.end()) - load.begin();
			pos = vChannels.begin();
			for (int i=0; i<j; i++) pos++;
			unsigned int k = *pos;
			assert(k>0);

			/* assign k to the link */
			T->AssignChannel2Link(e->second, k );
		}
		else if ( (CommonChannels.empty()) && (uChannels.size()==u->Info()->numAvailRadios()) && (vChannels.size()<v->Info()->numAvailRadios()) )
		{
			vector<double> load(uChannels.size());
			for (vector<double>::iterator i=load.begin(); i!=load.end(); i++) *i=0.;

			/* find the load of the channels of u in PE[e] */
			set<unsigned int>::iterator pos;
			int j;
			for (set<TLinkItrs>::iterator l=e->first.begin(); l!=e->first.end(); l++)
				if ( (l->second->Radio() != NULL) && ((pos=uChannels.find(l->second->RadioChannel())) != uChannels.end()) )
				{
					j=0;
					for (set<unsigned int>::iterator temp=uChannels.begin(); temp!=pos; temp++)
						j++;
					load[j]+=l->second->Flow();
				}

			/* find the least used channel in PE[e] */
			j = min_element(load.begin(), load.end()) - load.begin();
			pos = uChannels.begin();
			for (int i=0; i<j; i++) pos++;
			unsigned int k = *pos;
			assert(k>0);

			/* assign k to the link */
			T->AssignChannel2Link(e->second, k );
		}
		else
		{
			vector<double> load(uChannels.size()+vChannels.size());
			for (vector<double>::iterator i=load.begin(); i!=load.end(); i++) *i=0.;

			/* find the load of the channels of u and v in PE[e] */
			set<unsigned int>::iterator pos;
			int j;
			for (set<TLinkItrs>::iterator l=e->first.begin(); l!=e->first.end(); l++)
				if (l->second->Radio() != NULL) 
				{
 					if ((pos=uChannels.find(l->second->RadioChannel())) != uChannels.end())
					{
						j=0;
						for (set<unsigned int>::iterator temp=uChannels.begin(); temp!=pos; temp++)
							j++;
						load[j]+=l->second->Flow();
					}
					else if ((pos=vChannels.find(l->second->RadioChannel())) != vChannels.end())
					{
						j=uChannels.size();
						for (set<unsigned int>::iterator temp=vChannels.begin(); temp!=pos; temp++)
							j++;
						load[j]+=l->second->Flow();
					}
				}

			/* find the least used channel in PE[e] */
			unsigned int k,k1;
			j = min_element(load.begin(), load.end()) - load.begin();
			if ((unsigned int)j<uChannels.size())
			{
				pos = uChannels.begin();
				for (int i=0; i<j; i++) pos++;
				k = *pos;
				/* find the most used channel in vChannels */
				vector<double>::iterator temp=load.begin();
				for (unsigned int i=0; i<uChannels.size(); i++) temp++;
				j = max_element(temp, load.end()) - temp;
				pos = vChannels.begin();
				for (int i=0; i<j; i++) pos++;
				k1 = *pos;
				replace(T,e->second,v->Id(),k,k1);
			}
			else
			{
				pos = vChannels.begin();
				for (int i=uChannels.size(); i<j; i++) pos++;
				k = *pos;
				/* find the most used channel in uChannels */
				vector<double>::iterator temp=load.begin();
				for (unsigned int i=0; i<uChannels.size(); i++) temp++;
				j = max_element(load.begin(), temp) - load.begin();
				pos = uChannels.begin();
				for (int i=0; i<j; i++) pos++;
				k1 = *pos;
				replace(T,e->second,u->Id(),k,k1);
			}
		}
	}

	return 0;
}

/* estimates the capacity of each link and store it both in availBW and flow */
double LACA::EstimateCapacity(Topology *T, TLinkItrs l)
{
	double sumflows = 0.;

	vector<TLinkItrs> Dcoll = T->computeNeighborhood(l);

	for (vector<TLinkItrs>::iterator i=Dcoll.begin(); i!=Dcoll.end(); i++)
		if (i->second->RadioChannel() == l.second->RadioChannel())
			sumflows += i->second->Flow();

	if (sumflows)
		return l.second->Flow() / sumflows * l.second->Rate();
	else
		return 0.;
}



void LACA::RanGopChi(Topology *T, int NumFlows, RndVar *Source, RndVar *Dest, RndVar *Bwdr, double seed)
{
	double CSgoodput;
	int curiter = 0, maxiter = 10;

	do
	{
		LACA laca;
		laca.Run(T);

		// estimate capacity for every link
		for (int u=0; u<T->GetNumNodes(); u++)
			for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++)
				SetAvailBW (v, EstimateCapacity(T,T->LinkItrs(T->Node(u),v)));

		// store the estimated capacity in flow
		for (int u=0; u<T->GetNumNodes(); u++)
			for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++)
				SetFlow (v, 2 * v->AvailBW());

		CSgoodput = T->RouteFlows(NumFlows, Source, Dest, Bwdr, seed);
	
		// compute the allocated bandwidth, which serves as the new flow
		for (int u=0; u<T->GetNumNodes(); u++)
			for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++)
				SetFlow (v, v->AvailBW());

		curiter++;

	} while (T->getUtotMax().second > 1. && curiter < maxiter);

	T->scaleFlowsByUtotMax();
}


