#include "milp.h"
#include <iostream>
#include <sstream>
#include <algorithm>
#include <numeric>
#include <fcntl.h>
// #include <glpk.h>
#include <CbcBranchDynamic.hpp>
#include <ClpSimplex.hpp>
#include <CoinBuild.hpp>




/** MILP methods **/

// void MILP::set_params ()
// {
//   if (timeout > 0)
//     set_timeout(lp, timeout);
// 
// //  set_epslevel(lp, EPS_BAGGY);
// //  set_epsel(lp, 1.0e-4);
// //  set_epsb(lp, 1.0e-5);
// //  set_epsd(lp, 1.0e-4);
// //  set_epsint(lp, 1.0e-3);
// //  set_break_at_first(lp, TRUE);
// 
// //  set_mip_gap(lp, FALSE, 0.05);
// //  set_break_at_value(lp, u0); 
// //  set_presolve(lp, PRESOLVE_ROWS | PRESOLVE_COLS | PRESOLVE_LINDEP, get_presolveloops(lp));
// }


// void MILP::saveMPS (char* filename)
// {
//   write_freemps(lp, filename);  
// }


int MILP::lp_solve ()
{
  CbcModel* model;
  bool done = false;
//     const char * argv2[]={"-solve"};
//     write_freemps(lp, "mps.tmp");

  while (!done)
  {
//       OsiClpSolverInterface solver1;
//       solver1.readMps("mps.tmp","");

//       model = new CbcModel(solver1);
    model = new CbcModel(*solver);

    if (timeout > 0)
      model->setMaximumSeconds (timeout);

    if (hasBreakValue)
    {
      model->setCutoff(breakValue);
      model->setMaximumSolutions(1);
    }

    if (allowedGap > 0.)
      model->setAllowableGap(allowedGap);

    callCbc("-solve", *model);
    
    //CbcMain (0,NULL,*model);

    if (hasBreakValue && model->isProvenInfeasible())
    {
      // make another iteration without break value
      hasBreakValue = false;
      delete model;
    }
    else
      done = true;
  }

  if (model->bestSolution())
  {
    obj_value = model->getObjValue();
    bound = model->getBestPossibleObjValue();

    if (apply)
      /* update the flows and the e2e flows and the Utot */
      updateFlows(model->getColSolution(), model->getRowPrice());

    delete model;

    return 0;
  }

  // no solution found
  cerr << endl << "WARNING: A (sub-)optimal solution to the MILP has NOT been found" << endl << endl;

  delete model;

  return -1;
}


int MILP::lag_solve (const set<int>& torelax, double mu_0, int max_iter, sub_grad_method* sgm)
{
//   set_params();
  relaxedConstrGroups.clear();

  set<int> relax_constr;

  for (set<int>::const_iterator s=torelax.begin(); s!=torelax.end(); s++)
  {
    // check if the group number is valid
    if (*s < 1 || *s > constrGroupSize.size()-1)
    {
      cerr << endl << "** WARNING: constraint group " << *s << " does not exist! **" << endl << endl;
      continue;
    }
    
    for (int i=constrGroupSize[(*s)-1]+1; i<=constrGroupSize[*s]; i++)
      relax_constr.insert(i);

    relaxedConstrGroups.insert(*s);
  }
  
//   Lag_Relax lag(lp, relax_constr, mu_0, max_iter);
  Lag_Relax lag(solver->getModelPtr(), relax_constr, mu_0, max_iter);

  if (sgm)
    lag.set_sgm(sgm);

//   lag.set_solver(ms);

  if (hasBreakValue)
    lag.set_breakValue(breakValue);
  
  lag.solveLAG();

  if (!lag.haveSolution())
    cerr << endl << "WARNING: A (sub-)optimal solution to the MILP has NOT been found" << endl << endl;
  else
  {
    obj_value = lag.getObjValue();

    bound = lag.getBound();

    if (apply)
      /* update the flows and the e2e flows and the Utot */
      updateFlows(lag.getSolution());
  }
  
  return (int)(lag.haveSolution());
}


MILP::~MILP ()
{
  /* clean up such that all used memory by lpsolve is freed */
//   if (lp)
//     delete_lp(lp);

  if (solver)
    delete solver;

  delete Tcopy;
}


/** MinPower_MILP methods **/
/* NOTE This program does not handle radios with different power consumption profiles.
   Hence, the power consumption profile of the first radio on a node is considered for
   all the radios of that node */

MinPower_MILP::MinPower_MILP(Topology* _T, double _maxUtot, vector<float>* nodePowerConstr, long _timeout, bool _apply)
  : MILP(_T, _timeout, _apply),
    maxUtot(_maxUtot)
{
  ClpSimplex* model = new ClpSimplex;
  int Ncol;
  double F = 0.;      // sum of all the demands

  Tcopy->ResetChannels();

  /* check that e2e flows are consistent and obtain the set of demands */
  if (!(Tcopy->checkE2Eflows(&demands)))
    cerr << endl << "WARNING: MinPower_MILP: Checking E2E flows before beginning FAILED" << endl << endl;

  // compute the number of demands and the sum of all their flows
  numDemands = demands.size();
  
  for (map<pair<int,int>,double>::iterator d=demands.begin(); d!=demands.end(); d++)
    F += d->second;

  cout << "The sum of the flows of all the demands is " << F << endl;

  // compute the cumulative number of neighbors for each node. It is such that:
  // - the number of outgoing links of node i is cumNumNeighbs[i+1] - cumNumNeighbs[i]
  // - the last element of cumNumNeighbs is the total number of links
  unsigned int cumNumNeighbs[Tcopy->GetNumNodes() + 1];

  cumNumNeighbs[0] = 0;
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
  	  cumNumNeighbs[u + 1] = cumNumNeighbs[u] + Tcopy->Node(u)->GetNumNeighbors();

  // the number of variables is numDemands*numLinks*numChannels (f^ab(u->v)[c]) + numNodes*numChannels (x^c(u)) + numNodes (y_u)
  Ncol = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + Tcopy->GetNumNodes();

  // create the LP problem
//   if ((lp = make_lp(0, Ncol)) == NULL)
//     throw -1;

  model->resize(0, Ncol);

  /* Columns (variables) are ordered this way:
      f_e1,d1,c1 ... f_e1,d1,cC ... f_e1,dD,c1 ... f_e1,dD,cC ... f_eE,dD,c1 ... f_eE,dD,cC  x^c1(n1) ... x^cC(n1) ... x^c1(nN) ... x^cC(nN)  y(n1) ... y(nN)
      where links e1 ... eE are given by
      neighbors(node 0) neighbors(node 1) ... neighbors(node N)
      and demands d1 ... dD are given in the order they are present in demands
  */

  /* give a name to the variables */
  int linkId = 0;
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) {
      int i=0;
      for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, i++)
	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	  stringstream s;
	  s << "f(" << u << "," << v->Id() << ")[" << c << "]{" << d->first.first << d->first.second << "}";
// 	  set_col_name(lp, (linkId*numDemands+i)*WMS.numAvailChannels()+c, (char*)s.str().c_str());
	  string str = s.str();
	  model->setColumnName((linkId*numDemands+i)*WMS.numAvailChannels()+c-1, str);
	}
      linkId++;
    }

  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++)
    for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      stringstream s;
      s << "x_" << i << "[" << c << "]";
      int j = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() + c;
//       set_col_name(lp, j, (char*)s.str().c_str());
// 	set_binary(lp, j, TRUE);
      string str = s.str();
      model->setColumnName(j-1, str);
      model->setInteger(j-1);
      model->setColumnBounds(j-1, 0., 1.);
    }

  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++)
  {
    stringstream s;
    s << "y_" << i;
    int j = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + i + 1;
//     set_col_name(lp, j, (char*)s.str().c_str());
//       set_binary(lp, j, TRUE);
    string str = s.str();
    model->setColumnName(j-1, str);
    model->setInteger(j-1);
    model->setColumnBounds(j-1, 0., 1.);
  }


  int* colno = new int[Ncol];
  double *coeff = new double[Ncol], *coeff2 = new double[Ncol];

  vector<vector<double> > coeff_tau;
  vector<vector<int> > colno_tau;

  vector<vector<double> > coeff_power;
  vector<vector<int> > colno_power;
  vector<double> lhs_power;

  map<int, set<TLinkItrs> > Linc;  // maps each node to the set of incoming links

  /* compute Linc for all the nodes */
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++)
      Linc[v->Id()].insert(Tcopy->LinkItrs(Tcopy->Node(u),v));

  int k, pos_i;

  /* With one iteration over all the nodes, we determine the coefficients used for the objective
   * function, the constraints tau_TX^(u,c) + tau_RX^(u,c) <= 1 (for each active radio) and node power constraints */
  for (unsigned int u=0; u<Tcopy->GetNumNodes(); u++)
  {
    k = 0;
    RadioIfaceIterator ri = Tcopy->Node(u)->Info()->Radio(0);

    /* for each channel */
//     for (unsigned int c=1; c<=WMS.numAvailChannels(); c++)
    for (unsigned int c=0; c<WMS.numAvailChannels(); c++)
    {
      int j1=k;

      for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++)
      {
	fill_n(coeff+k, numDemands, 1. / v->Rate());
	// coeff2 is used for the power consumption constraint
	fill_n(coeff2+k, numDemands, ((*ri->Rho_Tx())(DbmToW(WMS.TxPowerLevel(v->TxPower()))) - ri->Rho_Idle()) / v->Rate());
	pos_i = (cumNumNeighbs[u] + distance(Tcopy->Node(u)->NeighborBegin(),v))*numDemands*WMS.numAvailChannels() + c;
	generate_n(colno+k, numDemands, WMMILP::counter<int>(pos_i-WMS.numAvailChannels(), WMS.numAvailChannels()));
	k += numDemands;
      }

      int j2=k;

      if (Linc.find(u) != Linc.end())
	for (set<TLinkItrs>::iterator l=Linc[u].begin(); l!=Linc[u].end(); l++)
	{
	  fill_n(coeff+k, numDemands, 1. / l->second->Rate());
	  fill_n(coeff2+k, numDemands, (ri->Rho_Rx() - ri->Rho_Idle()) / l->second->Rate());
	  pos_i = (cumNumNeighbs[l->first->Id()] + distance(l->first->NeighborBegin(),l->second))*numDemands*WMS.numAvailChannels() + c;
	  generate_n(colno+k, numDemands, WMMILP::counter<int>(pos_i-WMS.numAvailChannels(), WMS.numAvailChannels()));
	  k += numDemands;
	}

      if (k != j1)
      {
	/* make a copy of coeff and colno so that later we can add the tau constraint */
	coeff_tau.push_back(vector<double>(coeff+j1,coeff+k));
	colno_tau.push_back(vector<int>(colno+j1,colno+k));
      }

      /* multiply coefficients by the power consumption (needed for node power constraint) */
//       for (int j=j1; j<j2; j++)
// 	coeff[j] *= (T->pm.rho_tx-T->pm.rho_idle);
// 
//       for (int j=j2; j<k; j++)
// 	coeff[j] *= (T->pm.rho_rx-T->pm.rho_idle);
    }

//     float tmp = T->pm.rho_idle-T->pm.rho_sleep;
    float tmp = ri->Rho_Idle() - ri->Rho_Sleep();

    /* stores the coefficients for the power constraint on node u */
    fill_n(coeff2+k, WMS.numAvailChannels(), tmp);      // x_u^c * (rho_idle - rho_sleep)
    generate_n(colno+k, WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + u*WMS.numAvailChannels() - 1));
    k += WMS.numAvailChannels();

//     tmp = T->pm.BasePowerConsumption + T->node[u].info->numAvailRadios() * T->pm.rho_sleep - T->pm.StandbyPowerConsumption;
    tmp = Tcopy->Node(u)->Info()->GetBasePowerConsumption() + Tcopy->Node(u)->Info()->numAvailRadios() * ri->Rho_Sleep() - Tcopy->Node(u)->Info()->GetStandbyPowerConsumption();

    coeff2[k] = tmp;
    colno[k++] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + u;   // y_u

    coeff_power.push_back(vector<double>(coeff2,coeff2+k));
    colno_power.push_back(vector<int>(colno,colno+k));
    lhs_power.push_back(nodePowerConstr->at(u) - Tcopy->Node(u)->Info()->GetStandbyPowerConsumption());
  }

  for (int i=0; i<Ncol; i++)
    coeff[i] = 0.;               // reset the elements of coeff

  for (unsigned int i=0; i<coeff_power.size(); i++)
    for (unsigned int j=0; j<coeff_power.at(i).size(); j++)
      coeff[colno_power.at(i).at(j)] += coeff_power.at(i).at(j);
    
  /* set the objective function: min sum_u\in V P(u) */
//   set_obj_fn(lp, coeff);

  for (int i=0; i<Ncol; i++)
    model->setObjectiveCoefficient(i, coeff[i]);

  /* set the object direction to minimize */
//   set_minim(lp);
  model->setOptimizationDirection(1);

//   set_add_rowmode(lp, TRUE);  /* makes building the model faster if it is done rows by row */

  CoinBuild buildObject;

  /* constraints on the total utilizations ( |E|*numChannels constraints in total) */
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) 
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
	k = 0;
	for (int x=0; x<Tcopy->GetNumNodes(); x++)
	  for (TGraphNeighborIterator y=Tcopy->Node(x)->NeighborBegin(); y!=Tcopy->Node(x)->NeighborEnd(); y++)
	    if ( Tcopy->LinkItrs(Tcopy->Node(x),y).PotInterferesWith(Tcopy->LinkItrs(Tcopy->Node(u),v)) ) {
	      fill(coeff+k, coeff+k+numDemands, 1. / y->Rate());
	      for (unsigned int i=0; i<numDemands; i++)
		colno[k+i] = ((cumNumNeighbs[x] + distance(Tcopy->Node(x)->NeighborBegin(),y))*numDemands +i)*WMS.numAvailChannels() + c;
	      k += numDemands;
	    }
// 	add_constraintex(lp, k, coeff, colno, LE, maxUtot);
	buildObject.addRow(k, colno, coeff, 0., maxUtot);
      }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  /* tau constraints */
  for (unsigned int r=0; r<coeff_tau.size(); r++)
  {
    copy (coeff_tau.at(r).begin(), coeff_tau.at(r).end(), coeff);
    copy (colno_tau.at(r).begin(), colno_tau.at(r).end(), colno);

//     add_constraintex(lp, coeff_tau.at(r).size(), coeff, colno, LE, 1);
    buildObject.addRow(coeff_tau.at(r).size(), colno, coeff, 0., 1.);
  }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  /* node power constraints */
  for (unsigned int i=0; i<coeff_power.size(); i++)
  {
    copy (coeff_power.at(i).begin(), coeff_power.at(i).end(), coeff);
    copy (colno_power.at(i).begin(), colno_power.at(i).end(), colno);

//     add_constraintex(lp, coeff_power.at(i).size(), coeff, colno, LE, nodePowerConstr->at(i) - T->pm.StandbyPowerConsumption);
//     buildObject.addRow(coeff_power.at(i).size(), colno, coeff, 0., nodePowerConstr->at(i) - T->pm.StandbyPowerConsumption);
    buildObject.addRow(coeff_power.at(i).size(), colno, coeff, 0., lhs_power.at(i));
  }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  for (unsigned int u=0; u<Tcopy->GetNumNodes(); u++)
  {
    /* constraints y_u >= x_u^c, for each channel c */
//     for (unsigned int c=1; c<=WMS.numAvailChannels(); c++)
    for (unsigned int c=0; c<WMS.numAvailChannels(); c++)
    {
      coeff[0] = 1.;
      colno[0] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + u*WMS.numAvailChannels() + c;     // x_u^c

      coeff[1] = -1.;
//       colno[1] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + T->node.size()*WMS.numAvailChannels() + u + 1;   // y_u
      colno[1] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + u;   // y_u

//       add_constraintex(lp, 2, coeff, colno, LE, 0.);
      buildObject.addRow(2, colno, coeff, -COIN_DBL_MAX, 0.);
    }
    
    /* constraint y_u <= sum_c x_u^c */
    fill_n(coeff, WMS.numAvailChannels(), -1.);
//     generate_n(colno, WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + u*WMS.numAvailChannels()));
    generate_n(colno, WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + u*WMS.numAvailChannels() - 1));

    coeff[WMS.numAvailChannels()] = 1.;
//     colno[WMS.numAvailChannels()] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + T->node.size()*WMS.numAvailChannels() + u + 1;   // y_u
    colno[WMS.numAvailChannels()] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + u;   // y_u

//     add_constraintex(lp, WMS.numAvailChannels()+1, coeff, colno, LE, 0.);
    buildObject.addRow(WMS.numAvailChannels()+1, colno, coeff, -COIN_DBL_MAX, 0.);
  }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  vector<vector<int> > colno_v(Tcopy->GetNumNodes()*numDemands);
  vector<vector<double> > coeff_v(Tcopy->GetNumNodes()*numDemands);

  /* flow conservation constraints  ( |V| * numDemands constraints in total)
   * We calculate them all together, in order to perform just a single iteration over all the network links
   * of course, more memory is needed
   * We also exploit this loop to set some variables to zero
   */
  unsigned int dist;

  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) {
      int d=0;
      for (FlowDemandsIterator di=demands.begin(); di!=demands.end(); di++, d++) 
// 	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
	// for (unsigned int d=0; d<numDemands; d++) {
	  dist = distance(Tcopy->Node(u)->NeighborBegin(), v);
	  /* flow leaving node u */
	  coeff_v[u*numDemands+d].push_back(1);
	  colno_v[u*numDemands+d].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  /* flow entering node v */
	  coeff_v[v->Id()*numDemands+d].push_back(-1);
	  colno_v[v->Id()*numDemands+d].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  /* if v is the source of the demand or u is the destination of the demand, f^d(u->v)[c] must be null */
	  if ( (u == di->first.second) || (v->Id() == di->first.first) )
// 	    set_upbo(lp, ((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c, 0.);
	    model->setColumnUpper(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c, 0.);
//	  else
//	    set_upbo(lp, ((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c, di->second);
        }
    }

  /* Add the flow conservation constraints one-by-one */
  k=0;           // index of the constraint to be added
  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++)
    for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++) {
      copy(coeff_v[k].begin(), coeff_v[k].end(), coeff);
      copy(colno_v[k].begin(), colno_v[k].end(), colno);

      /* if i is the source of the demand d */
      if (i == d->first.first)
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, d->second);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, d->second, d->second);
      /* if i is the destination of the demand d */
      else if (i == d->first.second)
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, -d->second);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, -d->second, -d->second);
      else
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, 0);
      buildObject.addRow(coeff_v[k].size(), colno, coeff, 0., 0.);
      k++;
    }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  /* capacity constraints  ( |E| constraints in total) */
  /* This set of constraints often make the LP infeasible 
   * because it is not possible to satisfy them while routing
   * the requested e2e demands.
   * It is anyway safe to remove them because the goal of minimizing M
   * will force to minimize the e2e flows
   */
#if 0
  for (vector<TGraphNode>::iterator u=T->node.begin(); u!=T->node.end(); u++)
    for (list<TGraphNeighbor>::iterator v=u->neighbor.begin(); v!=u->neighbor.end(); v++) {
      fill(coeff, coeff+numDemands, 1);
      for (int i=0; i<numDemands; i++)
	colno[i] = (cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i +1;
      
      add_constraintex(lp, numDemands, coeff, colno, LE, v->rate);
    }
#endif

  /* constraints on the number of radios per node (|V| constraints in total)*/
  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++) {
    fill(coeff, coeff+WMS.numAvailChannels(), 1);
//     generate(colno, colno+WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + i*WMS.numAvailChannels()));
    generate(colno, colno+WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() - 1));

//     add_constraintex(lp,WMS.numAvailChannels(), coeff, colno, LE, T->node[i].info->numAvailRadios());
    buildObject.addRow(WMS.numAvailChannels(), colno, coeff, 0., Tcopy->Node(i)->Info()->numAvailRadios());
  }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  colno_v.clear();
  colno_v.resize(Tcopy->GetNumNodes()*WMS.numAvailChannels());
  coeff_v.clear();
  coeff_v.resize(Tcopy->GetNumNodes()*WMS.numAvailChannels());

  /* constraints to make the flow null on all the links on u using channel c if x^c(u) is null  ( |V| * numChannels constraints in total)
   * We calculate them all together, in order to perform just a single iteration over all the network links
   * of course, more memory is needed
   */
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) {
      int d=0;
      for (FlowDemandsIterator di=demands.begin(); di!=demands.end(); di++, d++) 
	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	  dist = distance(Tcopy->Node(u)->NeighborBegin(), v);
	  /* flow leaving node u */
	  coeff_v[u*WMS.numAvailChannels()+c-1].push_back(1);
// 	  colno_v[u->info->id*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  colno_v[u*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c - 1);
	  /* flow entering node v */
	  coeff_v[v->Id()*WMS.numAvailChannels()+c-1].push_back(1);
// 	  colno_v[v->info->id*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  colno_v[v->Id()*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c - 1);
        }
    }

  /* Add the constraints one-by-one */
  k=0;           // index of the constraint to be added
  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++)
    for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      copy(coeff_v[k].begin(), coeff_v[k].end(), coeff);
      copy(colno_v[k].begin(), colno_v[k].end(), colno);

//       coeff[coeff_v[k].size()] = -F / lowbo * coeff_v[k].size();
      coeff[coeff_v[k].size()] = -F * coeff_v[k].size();
//       colno[coeff_v[k].size()] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() + c;
      colno[coeff_v[k].size()] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() + c - 1;

      if (coeff_v[k].size())
// 	add_constraintex(lp, coeff_v[k].size()+1, coeff, colno, LE, 0.);
	buildObject.addRow(coeff_v[k].size()+1, colno, coeff, -COIN_DBL_MAX, 0.);

      k++;
    }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  /* constraints to make flow on u->v[c] null if u and v do not use channel c (2*|E|*numChannels constraints in total) */
//   for (vector<TGraphNode>::iterator u=T->node.begin(); u!=T->node.end(); u++)
//     for (list<TGraphNeighbor>::iterator v=u->neighbor.begin(); v!=u->neighbor.end(); v++) 
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
// 	fill(coeff, coeff+numDemands, 1);
// 	coeff[numDemands] = -F;
// 	
// 	for (unsigned int i=0; i<numDemands; i++)
// 	  colno[i] = ((cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i)*WMS.numAvailChannels() + c;
// 	colno[numDemands] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + u->info->id*WMS.numAvailChannels() + c;
// 	
// 	add_constraintex(lp, numDemands+1, coeff, colno, LE, 0.);
// 	
// 	fill(coeff, coeff+numDemands, 1);
// 	coeff[numDemands] = -F;
// 	
// 	for (unsigned int i=0; i<numDemands; i++)
// 	  colno[i] = ((cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i)*WMS.numAvailChannels() + c;
// 	colno[numDemands] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + v->info->id*WMS.numAvailChannels() + c;
// 	
// 	add_constraintex(lp, numDemands+1, coeff, colno, LE, 0.);
//       }


//   set_add_rowmode(lp, FALSE);            /* rowmode should be turned off again when done building the model */

  model->addRows(buildObject);

  // the model is used to build the solver (which will own the pointer)
  solver = new OsiClpSolverInterface(model, true);

  // write_LP(lp, stdout);
  /* I only want to see important messages on screen while solving */
  // set_verbose(lp, IMPORTANT);

  /*  Before quitting, clear memory*/
  delete colno;
  delete coeff;
}


void MinPower_MILP::updateFlows (const double* ptr_var, const double* ptr_constr)
{
  bool updateUtot = false;

  // if the Utot constraints have not been relaxed, use their values to set the Utot fields
  if (ptr_constr && (relaxedConstrGroups.find(1) == relaxedConstrGroups.end()))
  {
    updateUtot = true;
    cout << endl << "*** updateUtot true ***" << endl;
  }

  // now that the solution is going to be applied, the original topology can be resetted
  T->ResetChannels();

  unsigned int linkId = 0;
  for (int u=0; u<T->GetNumNodes(); u++) {
    int N = T->Node(u)->GetNumNeighbors();
    TGraphNeighborIterator v=T->Node(u)->NeighborBegin();
    for (int n=0; n<N; n++) {
      bool parallel = false;
      // clear the flows on link u->v
      SetFlow (v, 0.);
      ClearFlowDemands (v);
      for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	double _f=0;
	map<pair<int,int>,double> _f_e2e;
	int i=0;
	
	for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, i++)
	  if (ptr_var[linkId*numDemands*WMS.numAvailChannels()+i*WMS.numAvailChannels() +c-1] > almost_null)
	    _f += (_f_e2e[pair<int,int>(d->first.first,d->first.second)] = ptr_var[linkId*numDemands*WMS.numAvailChannels()+i*WMS.numAvailChannels() +c-1]);

	if (_f) {
	  if (!parallel) {
	    SetFlow (v, _f);
	    for (FlowDemandsIterator fdi = _f_e2e.begin(); fdi != _f_e2e.end(); fdi++)
	      SetFlowDemand (v, fdi->first, fdi->second);
	    if (updateUtot)
	      SetUtot (v, ptr_constr[linkId*WMS.numAvailChannels()+c-1]);

	    T->AssignChannel2Link(T->LinkItrs(T->Node(u),v),c);

	    parallel = true;
	  }
	  else {
	    TGraphNeighbor tgn (v->Info(),v->Rate(),v->TxPower());
	    TGraphNeighborIterator z = AddNeighbor (T->Node(u), &tgn);
	    SetFlow (z, _f);
	    for (FlowDemandsIterator fdi = _f_e2e.begin(); fdi != _f_e2e.end(); fdi++)
	      SetFlowDemand (z, fdi->first, fdi->second);
	    if (updateUtot)
	      SetUtot (z, ptr_constr[linkId*WMS.numAvailChannels()+c-1]);

	    T->AssignChannel2Link(T->LinkItrs(T->Node(u),z),c);
	  }
	}
      }
      /* if the flow is null on the link (hence no channel has been assigned to it), remove it */
      if (!parallel) {
	TGraphNeighborIterator z = v++;
	EraseNeighbor (T->Node(u), z);
      }
      else
	v++;
      linkId++;
    }
  }
}
    

/*
 * Rearrange e2e flows of given demands to minimize the maximum Utot
 * while maximizing the number of radios turned off
 * based on a Mixed Integer Linear Program
 * Changes are applied only if the model is valid (M0 != 0)
 */
MRMU_MILP::MRMU_MILP(Topology* _T, long int _timeout, bool _apply): MILP(_T, _timeout, _apply)
{
//   lprec *lp;
  ClpSimplex* model = new ClpSimplex;
  double F = 0.;      // sum of all the demands
  int Nradios = T->countNumRadios();
//   pair<bool, unsigned int> ch;
  int ri;
  double alpha = 20.;    // upper bound on M0

  /* check that e2e flows are consistent and obtain the set of demands */
  map<pair<int,int>,double> demands;
  
  if (!(T->checkE2Eflows(&demands)))
    cerr << endl << "WARNING: mrmu: Checking E2E flows before beginning FAILED" << endl << endl;
  
  // compute the number of demands and the sum of all their flows
  unsigned int numDemands = demands.size();
  
  for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++)
    F += d->second;

  cout << "The sum of the flows of all the demands is " << F << endl;

  /* compute Lr for all the radios */
  for (unsigned int i=0; i<T->GetNumNodes(); i++)
    for (TGraphNeighborIterator v=T->Node(i)->NeighborBegin(); v!=T->Node(i)->NeighborEnd(); v++)
    {
//       ch = SearchChannel(v->info, v->radio->channel);
      ri = v->Info()->findRadioWithChannel(v->RadioChannel());
      /* insert the link in the sets of the two radios at the endpoints of the link */
      Lr[TRadio(i,distance(T->Node(i)->Info()->Radio(0),RadioIfaceIterator(v->Radio())))].insert(T->LinkItrs(T->Node(i),v));
//       if (ch.first)
      if (ri >= 0)
	Lr[TRadio(v->Id(), ri)].insert(T->LinkItrs(T->Node(i),v));
    }

  // compute the cumulative number of neighbors for each node. It is such that:
  // - the number of outgoing links of node i is cumNumNeighbs[i+1] - cumNumNeighbs[i]
  // - the last element of cumNumNeighbs is the total number of links
  cumNumNeighbs.resize(T->GetNumNodes() + 1);

  cumNumNeighbs[0] = 0;
  for (int u=0; u<T->GetNumNodes(); u++)
  	  cumNumNeighbs[u + 1] = cumNumNeighbs[u] + T->Node(u)->GetNumNeighbors();

  // Likewise, for the number of radios
  cumNumRadios.resize(T->GetNumNodes() + 1);

  cumNumRadios[0] = 0;
  for (int u=0; u<T->GetNumNodes(); u++)
  	  cumNumRadios[u + 1] = cumNumRadios[u] + T->Node(u)->Info()->numAvailRadios();


  // the number of variables is numDemands*numLinks (g^ab(u->v)) + numRadios*2 (yi e zi) + 1 (M0)
  Ncol = numDemands*cumNumNeighbs[T->GetNumNodes()] + 2*Nradios + 1;

  // create the LP problem
//   if ((lp = make_lp(0, Ncol)) == NULL)
//     throw -1;

  model->resize(0, Ncol);

  /* Columns (variables) are ordered this way:
      g_e1,d1 ... g_e1,dD ... g_eE,d1 ... g_eE,dD  y1 ... yN  z1 ... zN  M0
      where links e1 ... eE are given by
      neighbors(node 0) neighbors(node 1) ... neighbors(node N)
      and demands d1 ... dD are given in the order they are present in demands
  */

  /* give a name to the variables */
  int linkId = 0;
  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) {
      int i=1;
      for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, i++) {
	stringstream s;
	s << "g(" << u << "," << v->Id() << ")[" << d->first.first << "-" << d->first.second << "]";
// 	set_col_name(lp, linkId*numDemands+i, (char*)s.str().c_str());
	string str = s.str();
	model->setColumnName(linkId*numDemands+i-1, str);
      }
      linkId++;
    }
  for (unsigned int i=0; i<T->GetNumNodes(); i++)
    for (unsigned int r=0; r<T->Node(i)->Info()->numAvailRadios(); r++) {
      stringstream s1, s2;
      s1 << "y[" << i << "." << r << "]";
//       set_col_name(lp, numDemands*cumNumNeighbs[T->node.size()]+cumNumRadios[i]+r+1, (char*)s1.str().c_str());
      string str1 = s1.str();
      model->setColumnName(numDemands*cumNumNeighbs[T->GetNumNodes()]+cumNumRadios[i]+r, str1);
      s2 << "z[" << i << "." << r << "]";
//       set_col_name(lp, numDemands*cumNumNeighbs[T->node.size()]+Nradios+cumNumRadios[i]+r+1, (char*)s2.str().c_str());
      string str2 = s2.str();
      model->setColumnName(numDemands*cumNumNeighbs[T->GetNumNodes()]+Nradios+cumNumRadios[i]+r, str2);
    }
//   set_col_name(lp, Ncol, "M0");
  {
    string str("MO");
    model->setColumnName(Ncol-1, str);
  }

  
  for (int i=numDemands*cumNumNeighbs[T->GetNumNodes()] + Nradios + 1;  i<Ncol; i++)
//     set_binary(lp, i, TRUE);  // sets variable zi to binary
  {
    model->setInteger(i);  // sets variable zi to binary
    model->setColumnBounds(i, 0., 1.);
  }

  int* colno = new int[Ncol];
  double* coeff = new double[Ncol];

  /* set the objective function: max M0 - sum_i 1/N yi */
  fill(coeff, coeff+Nradios, -1./Nradios);
//   generate(colno, colno+Nradios, WMMILP::counter<int>(numDemands*cumNumNeighbs[T->node.size()]));
  generate(colno, colno+Nradios, WMMILP::counter<int>(numDemands*cumNumNeighbs[T->GetNumNodes()]-1));

  coeff[Nradios] = 1;
//   colno[Nradios] = Ncol;
  colno[Nradios] = Ncol-1;

//   set_obj_fnex(lp, Nradios+1, coeff, colno);
  for (int i=0; i<=Nradios; i++)
    model->setObjectiveCoefficient(colno[i], coeff[i]);

  /* set the object direction to maximize */
//   set_maxim(lp);
  model->setOptimizationDirection(-1);

//   set_add_rowmode(lp, TRUE);  /* makes building the model faster if it is done rows by row */

  vector<vector<int> > colno_v(T->GetNumNodes()*numDemands);
  vector<vector<double> > coeff_v(T->GetNumNodes()*numDemands);

  CoinBuild buildObject;

  /* flow conservation constraints  ( |V| * numDemands constraints in total)
   * We calculate them all together, in order to perform just a single iteration over all the network links
   * of course, more memory is needed
   * We also exploit this loop to set some variables to zero
   */
  unsigned int dist;

  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) {
      int d=0;
      for (FlowDemandsIterator di=demands.begin(); di!=demands.end(); di++, d++) {
      // for (unsigned int d=0; d<numDemands; d++) {
	dist = distance(T->Node(u)->NeighborBegin(), v);
	/* flow leaving node u */
	coeff_v[u*numDemands+d].push_back(1);
// 	colno_v[u->info->id*numDemands+d].push_back((cumNumNeighbs[u->info->id] + dist)*numDemands + d + 1);
	colno_v[u*numDemands+d].push_back((cumNumNeighbs[u] + dist)*numDemands + d);
	/* flow entering node v */
	coeff_v[v->Id()*numDemands+d].push_back(-1);
// 	colno_v[v->info->id*numDemands+d].push_back((cumNumNeighbs[u->info->id] + dist)*numDemands + d + 1);
	colno_v[v->Id()*numDemands+d].push_back((cumNumNeighbs[u] + dist)*numDemands + d);
	/* if v is the source of the demand or u is the destination of the demand, f(u->v)[d] must be null */
	if ( (u == di->first.second) || (v->Id() == di->first.first) )
// 	  set_upbo(lp, (cumNumNeighbs[u->info->id] + dist)*numDemands + d + 1, 0.);
	  model->setColumnUpper((cumNumNeighbs[u] + dist)*numDemands + d, 0.);
      }
    }

  /* Add the flow conservation constraints one-by-one */
  int k=0;           // index of the constraint to be added
  for (unsigned int i=0; i<T->GetNumNodes(); i++)
    for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, k++) {

      /* if i is the source of the demand d */
      if (i == d->first.first) {
	coeff_v[k].push_back(-d->second);
// 	colno_v[k].push_back(Ncol);
	colno_v[k].push_back(Ncol-1);
      }
      /* if i is the destination of the demand d */
      else if (i == d->first.second) {
	coeff_v[k].push_back(d->second);
// 	colno_v[k].push_back(Ncol);
	colno_v[k].push_back(Ncol-1);
      }

      copy(coeff_v[k].begin(), coeff_v[k].end(), coeff);
      copy(colno_v[k].begin(), colno_v[k].end(), colno);

//       add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, 0);
      buildObject.addRow(coeff_v[k].size(), colno, coeff, 0., 0.);
    }

  /* capacity constraints  ( |E| constraints in total) */
  /* This set of constraints often make the LP infeasible 
   * because it is not possible to satisfy them while routing
   * the requested e2e demands.
   * It is anyway safe to remove them because of the
   * constraints on the total utilizations
   */
#if 0
  for (vector<TGraphNode>::iterator u=T->node.begin(); u!=T->node.end(); u++)
    for (list<TGraphNeighbor>::iterator v=u->neighbor.begin(); v!=u->neighbor.end(); v++) {
      fill(coeff, coeff+numDemands, 1);
      for (int i=0; i<numDemands; i++)
	colno[i] = (cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i +1;
      
      add_constraintex(lp, numDemands, coeff, colno, LE, v->rate);
    }
#endif

  /* constraints on the total utilizations ( |E| constraints in total) */
  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) {
      k = 0;
      for (int x=0; x<T->GetNumNodes(); x++)
	for (TGraphNeighborIterator y=T->Node(x)->NeighborBegin(); y!=T->Node(x)->NeighborEnd(); y++)
	  if ( T->LinkItrs(T->Node(x),y).IsInTheCollDomainOf(T->LinkItrs(T->Node(u),v)) ) {
	    fill(coeff+k, coeff+k+numDemands, 1. / y->Rate());
	    for (unsigned int i=0; i<numDemands; i++)
// 	      colno[k+i] = (cumNumNeighbs[x->info->id] + distance(x->neighbor.begin(),y))*numDemands +i +1;
	      colno[k+i] = (cumNumNeighbs[x] + distance(T->Node(x)->NeighborBegin(),y))*numDemands +i;
	    k += numDemands;
	  }
//       add_constraintex(lp, k, coeff, colno, LE, 1. );
      buildObject.addRow(k, colno, coeff, -COIN_DBL_MAX, 1.);
    }

  /* constraints to force a null flow on the radios turned off (Nradios constraints in total) */
  for (unsigned int i=0; i<T->GetNumNodes(); i++)
    for (unsigned int r=0; r<T->Node(i)->Info()->numAvailRadios(); r++)
      /* if this radio is used by some links */
      if (Lr.find(TRadio(i,r)) != Lr.end()) {
	k = 0;     // counts the variables
	/* for each link, add the variables for all the demands */
	for (set<TLinkItrs>::iterator l=Lr[TRadio(i,r)].begin(); l!=Lr[TRadio(i,r)].end(); l++, k+=numDemands) {
	  fill(coeff+k, coeff+k+numDemands, 1);
// 	  generate(colno+k, colno+k+numDemands, 
// 		   WMMILP::counter<int>((cumNumNeighbs[l->first->info->id]+distance(l->first->neighbor.begin(),l->second))*numDemands) );
	  generate(colno+k, colno+k+numDemands, 
		   WMMILP::counter<int>((cumNumNeighbs[l->first->Id()]+distance(l->first->NeighborBegin(),l->second))*numDemands-1) );
	}
	coeff[k] = -F * Lr[TRadio(i,r)].size();
// 	colno[k++] = numDemands*cumNumNeighbs[T->node.size()]+cumNumRadios[i]+r+1;    // yi
	colno[k++] = numDemands*cumNumNeighbs[T->GetNumNodes()]+cumNumRadios[i]+r;      // yi
// 	add_constraintex(lp, k, coeff, colno, LE, 0.);
	buildObject.addRow(k, colno, coeff, -COIN_DBL_MAX, 0.);
      }
      else {
	/* if no link uses this radio, it can be considered off (yi = 0, zi = 0) */
// 	set_bounds(lp, numDemands*cumNumNeighbs[T->node.size()]+cumNumRadios[i]+r+1, 0., 0. );            // yi
	model->setColumnBounds(numDemands*cumNumNeighbs[T->GetNumNodes()]+cumNumRadios[i]+r, 0., 0.);         // yi
// 	set_bounds(lp, numDemands*cumNumNeighbs[T->node.size()]+Nradios+cumNumRadios[i]+r+1, 0., 0. );    // zi
	model->setColumnBounds(numDemands*cumNumNeighbs[T->GetNumNodes()]+Nradios+cumNumRadios[i]+r, 0., 0.);       // zi
      }

  /* constraints to make yi variables binary (3*Nradios constraints in total) */
//   for (int r=1; r<=Nradios; r++) {
  for (int r=0; r<Nradios; r++) {
    /* yi - alpha*zi <= 0 */
    coeff[0] = 1;
    colno[0] = numDemands*cumNumNeighbs[T->GetNumNodes()] + r;  // yi
    
    coeff[1] = -alpha;
    colno[1] = numDemands*cumNumNeighbs[T->GetNumNodes()] + Nradios + r;  // zi

//     add_constraintex(lp, 2, coeff, colno, LE, 0.);
    buildObject.addRow(2, colno, coeff, -COIN_DBL_MAX, 0.);
    
    /* yi - M0 - alpha*zi >= -alpha */
    coeff[0] = 1;
    colno[0] = numDemands*cumNumNeighbs[T->GetNumNodes()] + r;  // yi
    
    coeff[1] = -1;
    colno[1] = Ncol;   // M0
    
    coeff[2] = -alpha;
    colno[2] = numDemands*cumNumNeighbs[T->GetNumNodes()] + Nradios + r;  // zi

//     add_constraintex(lp, 3, coeff, colno, GE, -alpha);
    buildObject.addRow(3, colno, coeff, -alpha, COIN_DBL_MAX);

    /* yi - M0 + alpha*zi <= alpha */
    coeff[0] = 1;
    colno[0] = numDemands*cumNumNeighbs[T->GetNumNodes()] + r;  // yi

    coeff[1] = -1;
    colno[1] = Ncol;   // M0

    coeff[2] = alpha;
    colno[2] = numDemands*cumNumNeighbs[T->GetNumNodes()] + Nradios + r;  // zi

//     add_constraintex(lp, 3, coeff, colno, LE, alpha);
    buildObject.addRow(3, colno, coeff, -COIN_DBL_MAX, alpha);
  }
  

//   set_add_rowmode(lp, FALSE);            /* rowmode should be turned off again when done building the model */

  model->addRows(buildObject);

  // the model is used to build the solver (which will own the pointer)
  solver = new OsiClpSolverInterface(model, true);

//   write_LP(lp, stdout);
  /* I only want to see important messages on screen while solving */
  // set_verbose(lp, IMPORTANT);

  /*  Before quitting, clear memory*/
  delete colno;
  delete coeff;
}

void MRMU_MILP::updateFlows(const double* ptr_var, const double* ptr_constr)
{
//   get_ptr_variables(lp, &ptr_var);
//   get_ptr_constraints(lp, &ptr_constr);
  
  /* If M0 == 0, the solution is invalid */
//   if (ptr_var[Ncol-1]) {
    int ri;
    unsigned int linkId = 0;
    int Nradios = T->countNumRadios();

    for (int u=0; u<T->GetNumNodes(); u++)
      for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++, linkId++)
      {
	SetFlow (v, 0.);
	ClearFlowDemands (v);

	int i=0;
	for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, i++)
	  if (ptr_var[linkId*numDemands + i])
	  {
	    SetFlowDemand (v, d->first, ptr_var[linkId*numDemands + i] / ptr_var[Ncol-1]);
	    SetFlow (v, v->Flow() + ptr_var[linkId*numDemands + i] / ptr_var[Ncol-1]);
	  }

	SetUtot (v, ptr_constr[T->GetNumNodes()*numDemands+linkId] / ptr_var[Ncol-1]);
      }

    /* turn off the radios with zi=0 */
    for (unsigned int i=0; i<T->GetNumNodes(); i++)
      for (unsigned int r=0; r<T->Node(i)->Info()->numAvailRadios(); r++)
	if (!ptr_var[numDemands*cumNumNeighbs[T->GetNumNodes()]+Nradios+cumNumRadios[i]+r])
	{
	  if (Lr.find(TRadio(i,r)) != Lr.end())
	    /* for all the links */
	    for (set<TLinkItrs>::iterator l=Lr[TRadio(i,r)].begin(); l!=Lr[TRadio(i,r)].end(); l++) {
	      if (l->second->Flow())
		cerr << "WARNING: mrmu is removing a link with non-null flow!" << endl;
	      /* first, remove the link from the Lr set of the other radio (not the extracted one) */
// 	      ch = SearchChannel(l->second->info, l->second->radio->channel);
	      ri = l->second->Info()->findRadioWithChannel(l->second->RadioChannel());
	      if (i == l->second->Id())
		Lr[TRadio(l->first->Id(), distance(l->first->Info()->Radio(0),RadioIfaceIterator(l->second->Radio())))].erase(*l);
	      else if (ri >= 0)
		Lr[TRadio(l->second->Id(), ri)].erase(*l);
	      /* then, remove the link from the topology*/
	      EraseNeighbor (T->Node(l->first->Id()), l->second);
	    }
	  /* switch off the radio (set channel to 0) */
	  SetChannel (T->Node(i)->Info()->Radio(r), 0);
	}
//   }
//   else
//     cerr << endl << "WARNING: the solution of the MILP is invalid (M0 = 0)" << endl << endl;

  /* clean up such that all used memory by lpsolve is freed */
//   if (lp)
//     delete_lp(lp);
  
  if (!(T->checkE2Eflows(NULL)))
    cerr << endl << "WARNING: mrmu: Checking E2E flows FAILED" << endl << endl;
}



/** MinMaxUtot_MILP methods **/
/* NOTE This program does not handle radios with different power consumption profiles.
   Hence, the power consumption profile of the first radio on a node is considered for
   all the radios of that node */

MinMaxUtot_MILP::MinMaxUtot_MILP(Topology* _T, vector<float> *nodePowerConstr, long _timeout, bool _apply)
  : MILP(_T, _timeout, _apply)
{
  ClpSimplex* model = new ClpSimplex;
  int Ncol;
  double F = 0.;      // sum of all the demands

  Tcopy->ResetChannels();

  /* check that e2e flows are consistent and obtain the set of demands */
  if (!(Tcopy->checkE2Eflows(&demands)))
    cerr << endl << "WARNING: MinMaxUtot_MILP: Checking E2E flows before beginning FAILED" << endl << endl;

  // compute the number of demands and the sum of all their flows
  numDemands = demands.size();
  
  for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++)
    F += d->second;

  cout << "The sum of the flows of all the demands is " << F << endl;

  // compute the cumulative number of neighbors for each node. It is such that:
  // - the number of outgoing links of node i is cumNumNeighbs[i+1] - cumNumNeighbs[i]
  // - the last element of cumNumNeighbs is the total number of links
  unsigned int cumNumNeighbs[Tcopy->GetNumNodes() + 1];

  cumNumNeighbs[0] = 0;
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
  	  cumNumNeighbs[u + 1] = cumNumNeighbs[u] + Tcopy->Node(u)->GetNumNeighbors();

  // the number of variables is numDemands*numLinks*numChannels (f^ab(u->v)[c]) + numNodes*numChannels (x^c(u)) + 1 (M)
  Ncol = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + 1;

  // if power constraints are supplied, we consider additional numNodes variables (y_u)
  if (nodePowerConstr)
    Ncol += Tcopy->GetNumNodes();

  // create the LP problem
//   if ((lp = make_lp(0, Ncol)) == NULL)
//     throw -1;

  model->resize(0, Ncol);
  
  /* Columns (variables) are ordered this way:
      f_e1,d1,c1 ... f_e1,d1,cC ... f_e1,dD,c1 ... f_e1,dD,cC ... f_eE,dD,c1 ... f_eE,dD,cC  x^c1(n1) ... x^cC(n1) ... x^c1(nN) ... x^cC(nN) [ y(n1) ... y(nN) ]  M
      where links e1 ... eE are given by
      neighbors(node 0) neighbors(node 1) ... neighbors(node N)
      and demands d1 ... dD are given in the order they are present in demands
  */

  /* give a name to the variables */
  int linkId = 0;
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) {
      int i=0;
      for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, i++)
	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	  stringstream s;
	  s << "f(" << u << "," << v->Id() << ")[" << c << "]{" << d->first.first << d->first.second << "}";
// 	  set_col_name(lp, (linkId*numDemands+i)*WMS.numAvailChannels()+c, (char*)s.str().c_str());
	  string str = s.str();
	  model->setColumnName((linkId*numDemands+i)*WMS.numAvailChannels()+c-1, str);
	}
      linkId++;
    }

  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++)
    for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      stringstream s;
      s << "x_" << i << "[" << c << "]";
      int j = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() + c;
//       set_col_name(lp, j, (char*)s.str().c_str());
// 	set_binary(lp, j, TRUE);
      string str = s.str();
      model->setColumnName(j-1, str);
      model->setInteger(j-1);
      model->setColumnBounds(j-1, 0., 1.);
    }

  if (nodePowerConstr)
    for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++)
    {
      stringstream s;
      s << "y_" << i;
//       int j = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + T->node.size()*WMS.numAvailChannels() + i + 1;
      int j = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + i;
//       set_col_name(lp, j, (char*)s.str().c_str());
// 	set_binary(lp, j, TRUE);
      string str = s.str();
      model->setColumnName(j, str);
      model->setInteger(j);
      model->setColumnBounds(j, 0., 1.);
    }

//   set_col_name(lp, Ncol, "M");
  {
    string str("M");
    model->setColumnName(Ncol-1, str);
  }

  int* colno = new int[Ncol];
  double *coeff = new double[Ncol], *coeff2 = new double[Ncol];
  
  /* set the objective function: min M*/
//   coeff[0] = 1;
//   colno[0] = Ncol;
//   set_obj_fnex(lp, 1, coeff, colno);

  model->setObjectiveCoefficient(Ncol-1, 1);

  /* set the object direction to minimize */
//   set_minim(lp);
  model->setOptimizationDirection(1);

//   set_add_rowmode(lp, TRUE);  /* makes building the model faster if it is done rows by row */

  vector<vector<int> > colno_v(Tcopy->GetNumNodes()*numDemands);
  vector<vector<double> > coeff_v(Tcopy->GetNumNodes()*numDemands);

  CoinBuild buildObject;

  /* flow conservation constraints  ( |V| * numDemands constraints in total)
   * We calculate them all together, in order to perform just a single iteration over all the network links
   * of course, more memory is needed
   * We also exploit this loop to set some variables to zero
   */
  unsigned int dist;

  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) {
      int d=0;
      for (FlowDemandsIterator di=demands.begin(); di!=demands.end(); di++, d++) 
// 	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
	// for (unsigned int d=0; d<numDemands; d++) {
	  dist = distance(Tcopy->Node(u)->NeighborBegin(), v);
	  /* flow leaving node u */
	  coeff_v[u*numDemands+d].push_back(1);
	  colno_v[u*numDemands+d].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  /* flow entering node v */
	  coeff_v[v->Id()*numDemands+d].push_back(-1);
	  colno_v[v->Id()*numDemands+d].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  /* if v is the source of the demand or u is the destination of the demand, f^d(u->v)[c] must be null */
	  if ( (u == di->first.second) || (v->Id() == di->first.first) )
// 	    set_upbo(lp, ((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c, 0.);
	    model->setColumnUpper(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c, 0.);
//	  else
//	    set_upbo(lp, ((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c, di->second);
        }
    }

  /* Add the flow conservation constraints one-by-one */
  int k=0;           // index of the constraint to be added
  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++)
    for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++) {
      copy(coeff_v[k].begin(), coeff_v[k].end(), coeff);
      copy(colno_v[k].begin(), colno_v[k].end(), colno);

      /* if i is the source of the demand d */
      if (i == d->first.first)
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, d->second);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, d->second, d->second);
      /* if i is the destination of the demand d */
      else if (i == d->first.second)
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, -d->second);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, -d->second, -d->second);
      else
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, 0);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, 0., 0.);
      k++;
    }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  /* capacity constraints  ( |E| constraints in total) */
  /* This set of constraints often make the LP infeasible 
   * because it is not possible to satisfy them while routing
   * the requested e2e demands.
   * It is anyway safe to remove them because the goal of minimizing M
   * will force to minimize the e2e flows
   */
#if 0
  for (vector<TGraphNode>::iterator u=T->node.begin(); u!=T->node.end(); u++)
    for (list<TGraphNeighbor>::iterator v=u->neighbor.begin(); v!=u->neighbor.end(); v++) {
      fill(coeff, coeff+numDemands, 1);
      for (int i=0; i<numDemands; i++)
	colno[i] = (cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i +1;
      
      add_constraintex(lp, numDemands, coeff, colno, LE, v->rate);
    }
#endif

  /* constraints on the total utilizations ( |E|*numChannels constraints in total) */
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) 
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
	k = 0;
	for (int x=0; x<Tcopy->GetNumNodes(); x++)
	  for (TGraphNeighborIterator y=Tcopy->Node(x)->NeighborBegin(); y!=Tcopy->Node(x)->NeighborEnd(); y++)
	    if ( Tcopy->LinkItrs(Tcopy->Node(x),y).PotInterferesWith(Tcopy->LinkItrs(Tcopy->Node(u),v)) ) {
	      fill(coeff+k, coeff+k+numDemands, 1. / y->Rate());
	      for (unsigned int i=0; i<numDemands; i++)
		colno[k+i] = ((cumNumNeighbs[x] + distance(Tcopy->Node(x)->NeighborBegin(),y))*numDemands +i)*WMS.numAvailChannels() + c;
	      k += numDemands;
	    }
	coeff[k] = -1;
// 	colno[k++] = Ncol;         // -M
	colno[k++] = Ncol-1;       // -M
// 	add_constraintex(lp, k, coeff, colno, LE, 0 );
	buildObject.addRow(k, colno, coeff, -COIN_DBL_MAX, 0.);
      }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  /* constraints on the number of radios per node (|V| constraints in total)*/
  for (unsigned int i=0; i<Tcopy->GetNumNodes(); i++) {
    fill(coeff, coeff+WMS.numAvailChannels(), 1);
//     generate(colno, colno+WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + i*WMS.numAvailChannels()));
    generate(colno, colno+WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() - 1));

//     add_constraintex(lp,WMS.numAvailChannels(), coeff, colno, LE, T->node[i].info->numAvailRadios());
    buildObject.addRow(WMS.numAvailChannels(), colno, coeff, 0., Tcopy->Node(i)->Info()->numAvailRadios());
  }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  /* constraints to make flow on u->v[c] null if u and v do not use channel c (2*|E|*numChannels constraints in total) */
  for (int u=0; u<Tcopy->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++) 
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
	fill(coeff, coeff+numDemands, 1);
	coeff[numDemands] = -F;

	for (unsigned int i=0; i<numDemands; i++)
	  colno[i] = ((cumNumNeighbs[u] + distance(Tcopy->Node(u)->NeighborBegin(),v))*numDemands +i)*WMS.numAvailChannels() + c;
	colno[numDemands] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + u*WMS.numAvailChannels() + c;

// 	add_constraintex(lp, numDemands+1, coeff, colno, LE, 0.);
	buildObject.addRow(numDemands+1, colno, coeff, -COIN_DBL_MAX, 0.);

	fill(coeff, coeff+numDemands, 1);
	coeff[numDemands] = -F;

	for (unsigned int i=0; i<numDemands; i++)
	  colno[i] = ((cumNumNeighbs[u] + distance(Tcopy->Node(u)->NeighborBegin(),v))*numDemands +i)*WMS.numAvailChannels() + c;
	colno[numDemands] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + v->Id()*WMS.numAvailChannels() + c;

// 	add_constraintex(lp, numDemands+1, coeff, colno, LE, 0.);
	buildObject.addRow(numDemands+1, colno, coeff, -COIN_DBL_MAX, 0.);
      }

//   constrGroupSize.push_back(get_Nrows(lp));
  constrGroupSize.push_back(buildObject.numberRows());

  if (nodePowerConstr)
  {
    for (unsigned int u=0; u<Tcopy->GetNumNodes(); u++)
    {
      /* constraints y_u >= x_u^c, for each channel c */
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++)
      for (unsigned int c=0; c<WMS.numAvailChannels(); c++)
      {
	coeff[0] = 1.;
	colno[0] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + u*WMS.numAvailChannels() + c;     // x_u^c

	coeff[1] = -1.;
// 	colno[1] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + T->node.size()*WMS.numAvailChannels() + u + 1;   // y_u
	colno[1] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + u;   // y_u

// 	add_constraintex(lp, 2, coeff, colno, LE, 0.);
	buildObject.addRow(2, colno, coeff, -COIN_DBL_MAX, 0.);
      }
      
      /* constraint y_u <= sum_c x_u^c */
      fill_n(coeff, WMS.numAvailChannels(), -1.);
//       generate_n(colno, WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + u*WMS.numAvailChannels()));
      generate_n(colno, WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + u*WMS.numAvailChannels() - 1));

      coeff[WMS.numAvailChannels()] = 1.;
//       colno[WMS.numAvailChannels()] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + T->node.size()*WMS.numAvailChannels() + u + 1;   // y_u
      colno[WMS.numAvailChannels()] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + u;   // y_u

//       add_constraintex(lp, WMS.numAvailChannels()+1, coeff, colno, LE, 0.);
      buildObject.addRow(WMS.numAvailChannels()+1, colno, coeff, -COIN_DBL_MAX, 0.);
    }

//     constrGroupSize.push_back(get_Nrows(lp));
    constrGroupSize.push_back(buildObject.numberRows());

    map<int, set<TLinkItrs> > Linc;  // maps each node to the set of incoming links

    /* compute Linc for all the nodes */
    for (int u=0; u<Tcopy->GetNumNodes(); u++)
      for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++)
	Linc[v->Id()].insert(Tcopy->LinkItrs(Tcopy->Node(u),v));

    int pos_i;
    /* constraints tau_TX^(u,c) + tau_RX^(u,c) <= 1 (for each channel) and node power constraints */
    for (unsigned int u=0; u<Tcopy->GetNumNodes(); u++)
    {
      k=0;
      RadioIfaceIterator ri = Tcopy->Node(u)->Info()->Radio(0);

      /* add the constraint tau_TX^(u,c) + tau_RX^(u,c) <= 1 for each active radio */
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++)
      for (unsigned int c=0; c<WMS.numAvailChannels(); c++)
      {
	int j1=k;

	for (TGraphNeighborIterator v=Tcopy->Node(u)->NeighborBegin(); v!=Tcopy->Node(u)->NeighborEnd(); v++)
	{
	  fill_n(coeff+k, numDemands, 1. / v->Rate());
	  // coeff2 is used for the power consumption constraint
	  fill_n(coeff2+k, numDemands, ((*ri->Rho_Tx())(DbmToW(WMS.TxPowerLevel(v->TxPower()))) - ri->Rho_Idle()) / v->Rate());
	  pos_i = (cumNumNeighbs[u] + distance(Tcopy->Node(u)->NeighborBegin(),v))*numDemands*WMS.numAvailChannels() + c;
	  generate_n(colno+k, numDemands, WMMILP::counter<int>(pos_i-WMS.numAvailChannels(), WMS.numAvailChannels()));
	  k += numDemands;
	}

	int j2=k;

	if (Linc.find(u) != Linc.end())
	  for (set<TLinkItrs>::iterator l=Linc[u].begin(); l!=Linc[u].end(); l++)
	  {
	    fill_n(coeff+k, numDemands, 1. / l->second->Rate());
	    fill_n(coeff2+k, numDemands, (ri->Rho_Rx() - ri->Rho_Idle()) / l->second->Rate());
	    pos_i = (cumNumNeighbs[l->first->Id()] + distance(l->first->NeighborBegin(),l->second))*numDemands*WMS.numAvailChannels() + c;
	    generate_n(colno+k, numDemands, WMMILP::counter<int>(pos_i-WMS.numAvailChannels(), WMS.numAvailChannels()));
	    k += numDemands;
	  }

	if (k != j1)
	{
	  /* make a copy of coeff and colno because add_constraintex sorts the element in the passed vectors */
	  double coeff_c[k-j1];
	  int colno_c[k-j1];

	  copy (coeff+j1, coeff+k, coeff_c);
	  copy (colno+j1, colno+k, colno_c);

// 	  add_constraintex(lp, k-j1, coeff_c, colno_c, LE, 1);
	  buildObject.addRow(k-j1, colno_c, coeff_c, 0., 1.);
	}

	/* multiply coefficients by the power consumption (needed for node power constraint) */
// 	for (int j=j1; j<j2; j++)
// 	  coeff[j] *= (T->pm.rho_tx-T->pm.rho_idle);
// 
// 	for (int j=j2; j<k; j++)
// 	  coeff[j] *= (T->pm.rho_rx-T->pm.rho_idle);
      }

//       float tmp = T->pm.rho_idle-T->pm.rho_sleep;
      float tmp = ri->Rho_Idle() - ri->Rho_Sleep();

      fill_n(coeff2+k, WMS.numAvailChannels(), tmp);      // x_u^c * (rho_idle - rho_sleep)
      generate_n(colno+k, WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + u*WMS.numAvailChannels() - 1));
      k += WMS.numAvailChannels();

//       tmp = T->pm.BasePowerConsumption + T->node[u].info->numAvailRadios() * T->pm.rho_sleep - T->pm.StandbyPowerConsumption;
      tmp = Tcopy->Node(u)->Info()->GetBasePowerConsumption() + Tcopy->Node(u)->Info()->numAvailRadios() * ri->Rho_Sleep() - Tcopy->Node(u)->Info()->GetStandbyPowerConsumption();

      coeff2[k] = tmp;
      colno[k++] = numDemands*cumNumNeighbs[Tcopy->GetNumNodes()]*WMS.numAvailChannels() + Tcopy->GetNumNodes()*WMS.numAvailChannels() + u;   // y_u

//       add_constraintex(lp, k, coeff, colno, LE, nodePowerConstr->at(u) - T->pm.StandbyPowerConsumption);
//       buildObject.addRow(k, colno, coeff, 0., nodePowerConstr->at(u) - T->pm.StandbyPowerConsumption);
      buildObject.addRow(k, colno, coeff2, 0., nodePowerConstr->at(u) - Tcopy->Node(u)->Info()->GetStandbyPowerConsumption());
    }

//     constrGroupSize.push_back(get_Nrows(lp));
    constrGroupSize.push_back(buildObject.numberRows());
  }

//   set_add_rowmode(lp, FALSE);            /* rowmode should be turned off again when done building the model */

  model->addRows(buildObject);

  // the model is used to build the solver (which will own the pointer)
  solver = new OsiClpSolverInterface(model, true);

  // write_LP(lp, stdout);
  /* I only want to see important messages on screen while solving */
  // set_verbose(lp, IMPORTANT);

  /*  Before quitting, clear memory*/
  delete colno;
  delete coeff;
}


void MinMaxUtot_MILP::updateFlows (const double* ptr_var, const double* ptr_constr)
{
  bool updateUtot = false;

  // TODO Utots could be updated it the utot constraints were the first in the program
  // if the Utot constraints have not been relaxed, use their values to set the Utot fields
//   if (ptr_constr && (relaxedConstrGroups.find(1) == relaxedConstrGroups.end()))
//   {
//     updateUtot = true;
//     cout << endl << "*** updateUtot true ***" << endl;
//   }

  // now that the solution is going to be applied, the original topology can be resetted
  T->ResetChannels();

  unsigned int linkId = 0;
  for (int u=0; u<T->GetNumNodes(); u++) {
    int N = T->Node(u)->GetNumNeighbors();
    TGraphNeighborIterator v=T->Node(u)->NeighborBegin();
    for (int n=0; n<N; n++) {
      bool parallel = false;
      // clear the flows on link u->v
      SetFlow (v, 0.);
      ClearFlowDemands (v);
      for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	double _f=0;
	map<pair<int,int>,double> _f_e2e;
	int i=0;
	
	for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, i++)
	  if (ptr_var[linkId*numDemands*WMS.numAvailChannels()+i*WMS.numAvailChannels() +c-1] > almost_null)
	    _f += (_f_e2e[pair<int,int>(d->first.first,d->first.second)] = ptr_var[linkId*numDemands*WMS.numAvailChannels()+i*WMS.numAvailChannels() +c-1]);

	if (_f) {
	  if (!parallel) {
	    SetFlow (v, _f);
	    for (FlowDemandsIterator fdi = _f_e2e.begin(); fdi != _f_e2e.end(); fdi++)
	      SetFlowDemand (v, fdi->first, fdi->second);
// 	    if (updateUtot)
// 	      v->Utot = ptr_constr[T->node.size()*numDemands+linkId*WMS.numAvailChannels()+c-1] + ptr_var[get_Ncolumns(lp)-1];

	    T->AssignChannel2Link(T->LinkItrs(T->Node(u),v),c);

	    parallel = true;
	  }
	  else {
	    TGraphNeighbor tgn (v->Info(),v->Rate(),v->TxPower());
	    TGraphNeighborIterator z = AddNeighbor (T->Node(u), &tgn);
	    SetFlow (z, _f);
	    for (FlowDemandsIterator fdi = _f_e2e.begin(); fdi != _f_e2e.end(); fdi++)
	      SetFlowDemand (z, fdi->first, fdi->second);
// 	    if (updateUtot)
// 	      z->Utot = ptr_constr[T->node.size()*numDemands+linkId*WMS.numAvailChannels()+c-1] + ptr_var[get_Ncolumns(lp)-1];

	    T->AssignChannel2Link(T->LinkItrs(T->Node(u),z),c);
	  }
	}
      }
      /* if the flow is null on the link (hence no channel has been assigned to it), remove it */
      if (!parallel) {
	TGraphNeighborIterator z = v++;
	EraseNeighbor (T->Node(u), z);
      }
      else
	v++;
      linkId++;
    }
  }
cout << endl << "*** exiting updateFlows ***" << endl;
}


/*
 * Assign channels and rearrange e2e flows of given demands to minimize the number
 * of radios turned on given a constraint on the maximum total utilization
 * Based on a Mixed Integer Linear Program
 * ResetChannels is called to avoid duplicate neighbors in TGraphNeighbor lists and
 * reset the channel of every radio to zero (they are set after solving the MILP)
 * Returns true if the model is infeasible, false otherwise
 * The current configuration of T is used as initial basis
 */
MinRadios_MILP::MinRadios_MILP (Topology* _T, double u0, long _timeout, bool _apply)
 : MILP (_T, _timeout, _apply)
{
//   lprec *lp;
  ClpSimplex* model = new ClpSimplex;
  int Ncol;
  double F = 0.;      // sum of all the demands
  bool infeasible = true;

  // clone the topology before resetting channels to save a feasible solution used as initial basis
  Topology* U = new Topology(*T);
  
  T->ResetChannels();
  
  /* check that e2e flows are consistent and obtain the set of demands */
  map<pair<int,int>,double> demands;
  
  if (!(T->checkE2Eflows(&demands)))
    cerr << endl << "WARNING: E2CAR_MILP: Checking E2E flows before beginning FAILED" << endl << endl;

  // compute the number of demands and the sum of all their flows
  unsigned int numDemands = demands.size();
  
  for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++)
    F += d->second;

  cout << "The sum of the flow of all the demands is " << F << endl;

  // compute the cumulative number of neighbors for each node. It is such that:
  // - the number of outgoing links of node i is cumNumNeighbs[i+1] - cumNumNeighbs[i]
  // - the last element of cumNumNeighbs is the total number of links
  unsigned int cumNumNeighbs[T->GetNumNodes() + 1];

  cumNumNeighbs[0] = 0;
  for (int u=0; u<T->GetNumNodes(); u++)
  	  cumNumNeighbs[u + 1] = cumNumNeighbs[u] + T->Node(u)->GetNumNeighbors();

  // the number of variables is numDemands*numLinks*numChannels (f^ab(u->v)[c]) + numNodes*numChannels (x^c(u))
  Ncol = numDemands*cumNumNeighbs[T->GetNumNodes()]*WMS.numAvailChannels() + T->GetNumNodes()*WMS.numAvailChannels();

  // initial feasible solution
//   REAL guessvector[1+Ncol];
//   for (int i=0; i<=Ncol; i++)
//     guessvector[i] = 0.0;
  
  // create the LP problem
//   if ((lp = make_lp(0, Ncol)) == NULL)
//     throw -1;
  model->resize(0, Ncol);

  /* Columns (variables) are ordered this way:
      f_e1,d1,c1 ... f_e1,d1,cC ... f_e1,dD,c1 ... f_e1,dD,cC ... f_eE,dD,c1 ... f_eE,dD,cC  x^c1(n1) ... x^cC(n1) ... x^c1(nN) ... x^cC(nN)  M
      where links e1 ... eE are given by
      neighbors(node 0) neighbors(node 1) ... neighbors(node N)
      and demands d1 ... dD are given in the order they are present in demands
  */

  /* give a name to the variables */
  int linkId = 0;
  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) {
      int di=0;
      for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, di++)
	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	  stringstream s;
	  s << "f(" << u << "," << v->Id() << ")[" << c << "]{" << d->first.first << d->first.second << "}";
// 	  set_col_name(lp, (linkId*numDemands+di)*WMS.numAvailChannels()+c, (char*)s.str().c_str());
	  string str = s.str();
	  model->setColumnName((linkId*numDemands+di)*WMS.numAvailChannels()+c-1, str);

	  // store the value of this variable in the feasible solution
	  // check if a link u->v on channel c exists
// 	  list<TGraphNeighbor>::iterator nb = U->find_neighbor(u->info->id, v->info->id, c);
// 	  // and has some flow of the current demand
// 	  if (nb != U->node[u->info->id].neighbor.end() && nb->f_e2e.find(d->first) != nb->f_e2e.end() )
// 	    guessvector[(linkId*numDemands+di)*WMS.numAvailChannels()+c] = nb->f_e2e[d->first];
	}
      linkId++;
    }

  // find the maximum number of radios per node
  int rpn = 0;
  for (int n=0; n<T->GetNumNodes(); n++)
    if (T->Node(n)->Info()->numAvailRadios() > rpn)
      rpn = T->Node(n)->Info()->numAvailRadios();

  double lowbo = rpn/(rpn+1.0)+0.01;

  for (int i=0; i<T->GetNumNodes(); i++)
    for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      stringstream s;
      s << "x_" << i << "[" << c << "]";
      int j = numDemands*cumNumNeighbs[T->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() + c;
//       set_col_name(lp, j, (char*)s.str().c_str());
      string str =s.str();
      model->setColumnName(j-1, str);
      model->setInteger(j-1);
      model->setColumnBounds(j-1, 0., 1.);
//       if (!relax)
// 	set_binary(lp, j, TRUE);
//       else
//       {
// 	set_semicont(lp, j, TRUE);
// 	set_lowbo(lp, j, lowbo);
// 	set_upbo(lp, j, 1.);      // sets xi <= 1
//       }

      // store the value of this variable in the feasible solution
//       if (U->node[i].info->countRadiosWithChannel(c))
// 	guessvector[j] = lowbo;
    }

  
  int* colno = new int[Ncol];
  double* coeff = new double[Ncol];

  /* set the objective function: min sum x^c(u) */
  fill(coeff, coeff+T->GetNumNodes()*WMS.numAvailChannels(), 1);
//   generate(colno, colno+T->node.size()*WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels()));
  generate(colno, colno+T->GetNumNodes()*WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->GetNumNodes()]*WMS.numAvailChannels()-1));

//   set_obj_fnex(lp, T->node.size()*WMS.numAvailChannels(), coeff, colno);
  for (int i=0; i<T->GetNumNodes()*WMS.numAvailChannels(); i++)
    model->setObjectiveCoefficient(colno[i], coeff[i]);

  /* set the object direction to minimize */
//   set_minim(lp);
  model->setOptimizationDirection(1);

//   set_epslevel(lp, EPS_BAGGY);
//   set_epsel(lp, 1.0e-2);
//   set_epsb(lp, 1.0e-5);
//   set_epsd(lp, 1.0e-4);
//   set_epsint(lp, 1.0e-3);
//   set_break_at_first(lp, TRUE);
//   set_timeout(lp, 1800);
  
//  set_scaling(lp, SCALE_MEAN | SCALE_EQUILIBRATE | SCALE_INTEGERS );

//    set_improve(lp, IMPROVE_DUALFEAS | IMPROVE_BBSIMPLEX);
    //set_basiscrash(lp, CRASH_LEASTDEGENERATE);
//    set_anti_degen(lp, ANTIDEGEN_FIXEDVARS | ANTIDEGEN_RHSPERTURB);
//  set_BFP(lp,"bfp_LUSOL");
  
//   if (relax)
//     set_mip_gap(lp, FALSE, 0.1);
  
   //set_presolve(lp, PRESOLVE_ROWS | PRESOLVE_COLS | PRESOLVE_LINDEP, get_presolveloops(lp));
   //default_basis(lp);
  
//   set_pivoting(lp, PRICER_FIRSTINDEX | PRICE_ADAPTIVE);
//    set_bb_rule(lp, NODE_PSEUDORATIOSELECT | NODE_AUTOORDER | NODE_BREADTHFIRSTMODE);
  
//   set_add_rowmode(lp, TRUE);  /* makes building the model faster if it is done rows by row */

  vector<vector<int> > colno_v(T->GetNumNodes()*numDemands);
  vector<vector<double> > coeff_v(T->GetNumNodes()*numDemands);

  CoinBuild buildObject;

  /* flow conservation constraints  ( |V| * numDemands constraints in total)
   * We calculate them all together, in order to perform just a single iteration over all the network links
   * of course, more memory is needed
   * We also exploit this loop to set some variables to zero
   */
  unsigned int dist;
  set<int> cancel;

  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) {
      int d=0;
      for (FlowDemandsIterator di=demands.begin(); di!=demands.end(); di++, d++) 
// 	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
	// for (unsigned int d=0; d<numDemands; d++) {
	  dist = distance(T->Node(u)->NeighborBegin(), v);
	  /* flow leaving node u */
	  coeff_v[u*numDemands+d].push_back(1);
	  colno_v[u*numDemands+d].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  /* flow entering node v */
	  coeff_v[v->Id()*numDemands+d].push_back(-1);
	  colno_v[v->Id()*numDemands+d].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  /* if v is the source of the demand or u is the destination of the demand, f^d(u->v)[c] must be null */
	  if ( (u == di->first.second) || (v->Id() == di->first.first) )
	    //cancel.insert(((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
// 	    set_upbo(lp, ((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c, 0.);
	    model->setColumnUpper(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c, 0.);
//	  else
//	    set_upbo(lp, ((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c, di->second);
        }
    }

  /* Add the flow conservation constraints one-by-one */
  int k=0;           // index of the constraint to be added
  for (unsigned int i=0; i<T->GetNumNodes(); i++)
    for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++) {
      copy(coeff_v[k].begin(), coeff_v[k].end(), coeff);
      copy(colno_v[k].begin(), colno_v[k].end(), colno);

      /* if i is the source of the demand d */
      if (i == d->first.first)
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, d->second);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, d->second, d->second);
      /* if i is the destination of the demand d */
      else if (i == d->first.second)
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, -d->second);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, -d->second, -d->second);
      else
// 	add_constraintex(lp, coeff_v[k].size(), coeff, colno, EQ, 0);
	buildObject.addRow(coeff_v[k].size(), colno, coeff, 0., 0.);
      k++;
    }

  /* capacity constraints  ( |E| constraints in total) */
  /* This set of constraints often make the LP infeasible 
   * because it is not possible to satisfy them while routing
   * the requested e2e demands.
   * It is anyway safe to remove them because the goal of minimizing M
   * will force to minimize the e2e flows
   */
#if 0
  for (vector<TGraphNode>::iterator u=T->node.begin(); u!=T->node.end(); u++)
    for (list<TGraphNeighbor>::iterator v=u->neighbor.begin(); v!=u->neighbor.end(); v++) {
      fill(coeff, coeff+numDemands, 1);
      for (int i=0; i<numDemands; i++)
	colno[i] = (cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i +1;
      
      add_constraintex(lp, numDemands, coeff, colno, LE, v->rate);
    }
#endif

  /* constraints on the total utilizations ( |E|*numChannels constraints in total) */
  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) 
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
      for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
	k = 0;
	for (int x=0; x<T->GetNumNodes(); x++)
	  for (TGraphNeighborIterator y=T->Node(x)->NeighborBegin(); y!=T->Node(x)->NeighborEnd(); y++)
	    if ( T->LinkItrs(T->Node(x),y).PotInterferesWith(T->LinkItrs(T->Node(u),v)) ) {
	      fill(coeff+k, coeff+k+numDemands, 1. / y->Rate());
	      for (unsigned int i=0; i<numDemands; i++)
		colno[k+i] = ((cumNumNeighbs[x] + distance(T->Node(x)->NeighborBegin(),y))*numDemands +i)*WMS.numAvailChannels() + c;
	      k += numDemands;
	    }
// 	add_constraintex(lp, k, coeff, colno, LE, u0 );
	buildObject.addRow(k, colno, coeff, -COIN_DBL_MAX, u0);
      }

  /* constraints on the number of radios per node (|V| constraints in total)*/
  for (int i=0; i<T->GetNumNodes(); i++) {
    fill(coeff, coeff+WMS.numAvailChannels(), 1);
//     generate(colno, colno+WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + i*WMS.numAvailChannels()));
    generate(colno, colno+WMS.numAvailChannels(), WMMILP::counter<int>(numDemands*cumNumNeighbs[T->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() - 1));

//     add_constraintex(lp,WMS.numAvailChannels(), coeff, colno, LE, T->node[i].info->numAvailRadios());
    buildObject.addRow(WMS.numAvailChannels(), colno, coeff, -COIN_DBL_MAX, T->Node(i)->Info()->numAvailRadios());
  }


  colno_v.clear();
  colno_v.resize(T->GetNumNodes()*WMS.numAvailChannels());
  coeff_v.clear();
  coeff_v.resize(T->GetNumNodes()*WMS.numAvailChannels());

  /* constraints to make the flow null on all the links on u using channel c if x^c(u) is null  ( |V| * numChannels constraints in total)
   * We calculate them all together, in order to perform just a single iteration over all the network links
   * of course, more memory is needed
   */
  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) {
      int d=0;
      for (FlowDemandsIterator di=demands.begin(); di!=demands.end(); di++, d++) 
	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	  dist = distance(T->Node(u)->NeighborBegin(), v);
	  /* flow leaving node u */
	  coeff_v[u*WMS.numAvailChannels()+c-1].push_back(1);
// 	  colno_v[u->info->id*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  colno_v[u*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c - 1);
	  /* flow entering node v */
	  coeff_v[v->Id()*WMS.numAvailChannels()+c-1].push_back(1);
// 	  colno_v[v->info->id*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u->info->id] + dist)*numDemands + d)*WMS.numAvailChannels() + c);
	  colno_v[v->Id()*WMS.numAvailChannels()+c-1].push_back(((cumNumNeighbs[u] + dist)*numDemands + d)*WMS.numAvailChannels() + c - 1);
        }
    }

  /* Add the constraints one-by-one */
  k=0;           // index of the constraint to be added
  for (unsigned int i=0; i<T->GetNumNodes(); i++)
//     for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
    for (unsigned int c=0; c<WMS.numAvailChannels(); c++) {
      copy(coeff_v[k].begin(), coeff_v[k].end(), coeff);
      copy(colno_v[k].begin(), colno_v[k].end(), colno);

      coeff[coeff_v[k].size()] = -F / lowbo * coeff_v[k].size();
      colno[coeff_v[k].size()] = numDemands*cumNumNeighbs[T->GetNumNodes()]*WMS.numAvailChannels() + i*WMS.numAvailChannels() + c;
      
      if (coeff_v[k].size())
// 	add_constraintex(lp, coeff_v[k].size()+1, coeff, colno, LE, 0.);
	buildObject.addRow(coeff_v[k].size()+1, colno, coeff, -COIN_DBL_MAX, 0.);
      
      k++;
    }
  
  
  
  /* constraints to make flow on u->v[c] null if u and v do not use channel c (2*|E|*numChannels constraints in total) */
//   for (vector<TGraphNode>::iterator u=T->node.begin(); u!=T->node.end(); u++)
//     for (list<TGraphNeighbor>::iterator v=u->neighbor.begin(); v!=u->neighbor.end(); v++) 
//       for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
// 	fill(coeff, coeff+numDemands, 1);
// 	coeff[numDemands] = (relax ? -F/lowbo : -F);
// 	
// 	for (unsigned int i=0; i<numDemands; i++)
// 	  colno[i] = ((cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i)*WMS.numAvailChannels() + c;
// 	colno[numDemands] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + u->info->id*WMS.numAvailChannels() + c;
// 	
// 	add_constraintex(lp, numDemands+1, coeff, colno, LE, 0.);
// 	
// 	fill(coeff, coeff+numDemands, 1);
// 	coeff[numDemands] = (relax ? -F/lowbo : -F);
// 	
// 	for (unsigned int i=0; i<numDemands; i++)
// 	  colno[i] = ((cumNumNeighbs[u->info->id] + distance(u->neighbor.begin(),v))*numDemands +i)*WMS.numAvailChannels() + c;
// 	colno[numDemands] = numDemands*cumNumNeighbs[T->node.size()]*WMS.numAvailChannels() + v->info->id*WMS.numAvailChannels() + c;
// 	
// 	add_constraintex(lp, numDemands+1, coeff, colno, LE, 0.);
//       }


//   set_add_rowmode(lp, FALSE);            /* rowmode should be turned off again when done building the model */

  model->addRows(buildObject);

  // the model is used to build the solver (which will own the pointer)
  solver = new OsiClpSolverInterface(model, true);

  // remove the variables that are known to be null
//   for (set<int>::iterator i=cancel.begin(); i!=cancel.end(); i++)
//     if (del_column(lp, *i) == FALSE)
//       cerr << endl << "WARNING: Cannot delete a column" << endl << endl;
  
//   int basis[1+Ncol+get_Nrows(lp)];
//   
//   if (guess_basis(lp, guessvector, basis) == TRUE)
//     cout << endl <<  "A valid base has been determined" << endl << endl;
//   else
//     cout << endl <<  "WARNING: A valid base could have not been determined" << endl << endl;
//   
//   if (set_basis(lp, basis, TRUE) == TRUE)
//     cout << endl <<  "The provided basis has been set" << endl << endl;
//   else
//     cout << endl <<  "WARNING: The provided basis could have not been set" << endl << endl;
  
  //write_LP(lp, stdout);
  /* I only want to see important messages on screen while solving */
  // set_verbose(lp, IMPORTANT);

  /*  Before quitting, clear memory*/
  delete colno;
  delete coeff;
}

void MinRadios_MILP::updateFlows(const double* ptr_var, const double* ptr_constr)
{
  /* Now let lpsolve calculate a solution */
//   int ret = solve(lp);
// 
//   if (ret != OPTIMAL)
//     cerr << endl << "WARNING: MinRadiosOpt: An optimal solution to the LP has NOT been found and hence this function made no changes to the topology" << endl << endl;
// 
//   print_solution(lp,4);

  /* If the solution is feasible (though suboptimal) apply the changes */
//   if (ret!=INFEASIBLE)  {
//     infeasible = false;
// 
//     REAL *ptr_var, *ptr_constr;
//     
//     get_ptr_variables(lp, &ptr_var);
//     get_ptr_constraints(lp, &ptr_constr);
    
    unsigned int linkId = 0;
    for (int u=0; u<T->GetNumNodes(); u++) {
      int N = T->Node(u)->GetNumNeighbors();
      TGraphNeighborIterator v=T->Node(u)->NeighborBegin();
      for (int n=0; n<N; n++) {
	bool parallel = false;
	// clear the flows on link u->v
	SetFlow (v, 0.);
	ClearFlowDemands (v);
	for (unsigned int c=1; c<=WMS.numAvailChannels(); c++) {
	  double _f=0;
	  map<pair<int,int>,double> _f_e2e;
	  int i=0;
	  
	  for (FlowDemandsIterator d=demands.begin(); d!=demands.end(); d++, i++)
	  {
	    stringstream s;
	    s << "f(" << u << "," << v->Id() << ")[" << c << "]{" << d->first.first << d->first.second << "}";
// 	    int col = get_nameindex(lp,(char*)s.str().c_str(),FALSE);
	    const vector<string>* names = solver->getModelPtr()->columnNames();
	    vector<string>::const_iterator ni = find (names->begin(), names->end(), s.str());
	    // if (ptr_var[linkId*numDemands*WMS.numAvailChannels()+i*WMS.numAvailChannels() +c-1] > almost_null)
// 	    if (col != -1 && ptr_var[col-1] > almost_null)
	    if (ni != names->end()) 
	    {
	      int col = distance(names->begin(), ni);
	      if (ptr_var[col] > almost_null)
	      _f += (_f_e2e[pair<int,int>(d->first.first,d->first.second)] = ptr_var[col]);
	    }
	  }

	  if (_f) {
	    if (!parallel) {
	    SetFlow (v, _f);
	    for (FlowDemandsIterator fdi = _f_e2e.begin(); fdi != _f_e2e.end(); fdi++)
	      SetFlowDemand (v, fdi->first, fdi->second);
	      // TODO v->Utot = ptr_constr[T->node.size()*numDemands+linkId*WMS.numAvailChannels()+c-1] + ptr_var[Ncol-1];

	      T->AssignChannel2Link(T->LinkItrs(T->Node(u),v),c);

	      parallel = true;
	    }
	    else {
	      TGraphNeighbor tgn (v->Info(),v->Rate(),v->TxPower());
	      TGraphNeighborIterator z = AddNeighbor (T->Node(u), &tgn);
	    SetFlow (z, _f);
	    for (FlowDemandsIterator fdi = _f_e2e.begin(); fdi != _f_e2e.end(); fdi++)
	      SetFlowDemand (z, fdi->first, fdi->second);
	      // TODO z->Utot = ptr_constr[T->node.size()*numDemands+linkId*WMS.numAvailChannels()+c-1] + ptr_var[Ncol-1];

	      T->AssignChannel2Link(T->LinkItrs(T->Node(u),z),c);
	    }
	  }
	}
	/* if the flow is null on the link (hence no channel has been assigned to it), remove it */
	if (!parallel) {
	  TGraphNeighborIterator z = v++;
	  EraseNeighbor (T->Node(u), z);
	}
	else
	  v++;
	linkId++;
      }
    }
//   }
//   else
//     cerr << endl << "WARNING: the solution of the MILP is infeasible" << endl << endl;  
  
  /*  Before quitting, clear memory*/
//   delete U;
// 
//   /* clean up such that all used memory by lpsolve is freed */
//   if (lp)
//     delete_lp(lp);
// 
//   return infeasible;
}



/** Lag_Relax methods **/

// Lag_Relax::Lag_Relax (lprec* lp, set<int> relax_constr, REAL mu_0, int max_iter, int max_noimprove)
//   : m_lp(lp),
Lag_Relax::Lag_Relax (ClpSimplex* model, set<int> relax_constr, double mu_0, int max_iter, int max_noimprove)
  : m_model(model),
    m_curr_iter(0),
    m_max_iter(max_iter),
    m_max_noimprove(max_noimprove),
    m_hasBreakValue(false),
    m_have_solution(false),
//     m_solution(new REAL[get_Ncolumns(m_lp)]),
    m_solution(new double[m_model->getNumCols()]),
    m_obj_val(0.),
    m_have_bound(false),
    m_sgm(new inverseK(this))
//     m_solver(LpSolve)
{
  int nzelem;
//   REAL* row = new REAL[1+get_Ncolumns(m_lp)];
  const double* objcoeff;
//   int* colno = new int[1+get_Ncolumns(m_lp)];
  double* row;
  int* colno;

  /* get the objective function */
//   if (!get_row(m_lp, 0, row))
//   {
//     cerr << "ERROR: Lag_Relax: Error while getting the objective function of the provided program" << endl << endl;
//     throw;
//   }
  objcoeff = model->getRowObjCoefficients();

  /* save the information in a lag_constraint object */
//   m_obj_func = vector<REAL>(row, row+1+get_Ncolumns(m_lp));
  m_obj_func = vector<double>(objcoeff, objcoeff+m_model->getNumCols());

  CoinPackedMatrix* mat = m_model->matrix();

  /* for each constraint to be relaxed */
  for (set<int>::iterator i=relax_constr.begin(); i!=relax_constr.end(); i++)
//     if (*i >= 1 && *i <= get_Nrows(m_lp))
    if (*i >= 0 && *i < m_model->getNumRows())
    {
//       /* get the corresponding row */
//       if ((nzelem = get_rowex(m_lp, *i, row, colno)) < 0 )
//       {
// 	cerr << "WARNING: Lag_Relax: Error while getting row (constraint) " << *i << " in the provided program" << endl << endl;
// 	continue;
//       }

      /* find the index of the first element of the required row. See "Detailed Description"
       * at http://www.coin-or.org/Doxygen/CoinAll/class_coin_packed_matrix.html */
      nzelem = 0;
      for (int j=0; j < *i; j++)
	nzelem += mat->getVectorSize(j);

      row = new double[mat->getVectorSize(*i)];
      colno = new int[mat->getVectorSize(*i)];

      copy (mat->getElements()+nzelem, mat->getElements()+nzelem+mat->getVectorSize(*i), row);
      copy (mat->getIndices()+nzelem, mat->getIndices()+nzelem+mat->getVectorSize(*i), colno);

      /* save the information in a lag_constraint object */
//       m_relax_constr.push_back(lag_constraint(row, colno, nzelem, get_constr_type(m_lp, *i), get_rh(m_lp, *i), get_constr_type(m_lp, *i)==GE ? -fabs(mu_0) : fabs(mu_0)));

      m_relax_constr.push_back(lag_constraint(row, colno, mat->getVectorSize(*i), m_model->rowLower()[*i], m_model->rowUpper()[*i], (m_model->rowLower()[*i] == 0. ? -fabs(mu_0) : fabs(mu_0))));

      delete row;
      delete colno;
    }
    else
      cerr << "WARNING: Lag_Relax: Cannot find row (constraint) " << *i << " in the provided program" << endl << endl;

  nzelem = 0;     // number of deleted constraints
  for (set<int>::iterator i=relax_constr.begin(); i!=relax_constr.end(); i++)
//     if (*i >= 1 && *i <= get_Nrows(m_lp))
    if (*i >= 0 && *i < m_model->getNumRows())
    {
      /* delete the row from the program */
//       del_constraint(m_lp, *i-nzelem);
      model->deleteRows(1, &(*i));
      nzelem++;
    }

//   delete row;
//   delete colno;
}


void Lag_Relax::set_sgm (sub_grad_method* sgm)
{
  delete m_sgm;
  m_sgm = sgm;
  m_sgm->set_lag(this);
}


void Lag_Relax::update_obj_func ()
{
//   REAL* row = new REAL[1+get_Ncolumns(m_lp)];
  double* row = new double[m_model->getNumCols()];

  // copy the original objective function into row
  copy(m_obj_func.begin(), m_obj_func.end(), row);

  // add the relaxed constraints
  for (vector<lag_constraint>::iterator r=m_relax_constr.begin(); r!=m_relax_constr.end(); r++)
  {
    for (unsigned int i=0; i<r->row.size(); i++)
    {
      row[r->colno[i]] += (r->row[i] * r->mu);
    }
  }

//   if (!set_obj_fn(m_lp, row))
//     cerr << "WARNING: Lag_Relax: Cannot set the objective function at iteration " << m_curr_iter << endl << endl;

  for (int i=0; i<m_model->getNumCols(); i++)
    m_model->setObjectiveCoefficient(i, row[i]);

  delete row;
}


void Lag_Relax::update_multipliers (bool failed, bool noimprove)
{
  double sum, theta;

  if (!failed)
  {
    /* compute Ax-b  */
    m_Ax_b.clear();
    for (vector<lag_constraint>::iterator r=m_relax_constr.begin(); r!=m_relax_constr.end(); r++)
    {
      sum = 0.;
      for (unsigned int i=0; i<r->row.size(); i++)
      {
	sum += r->row[i] * m_curr_solution[r->colno[i]-1];
      }
//       sum -= r->rhs;
      sum -= (r->lower == -COIN_DBL_MAX ? r->upper : r->lower);

      m_Ax_b.push_back(sum);
    }
  }

  // compute theta
  theta = m_sgm->theta(failed || noimprove);
  
  // update the multipliers
  vector<double>::iterator v = m_Ax_b.begin();
  for (vector<lag_constraint>::iterator r=m_relax_constr.begin(); r!=m_relax_constr.end(); r++, v++)
  {
    r->mu += theta * *v;

//     if (r->type == LE)
//       r->mu = max (r->mu, 0.);   // cannot be negative
//     if (r->type == GE)
//       r->mu = min (r->mu, 0.);   // cannot be positive

    if (r->lower == -COIN_DBL_MAX)
      r->mu = max (r->mu, 0.);   // cannot be negative
    if (r->upper == COIN_DBL_MAX)
      r->mu = min (r->mu, 0.);   // cannot be positive
  }
}

#if 0
// callback function used to retrieve the global bound found by glpk
void my_glpk_callback (glp_tree* tree, void* info)
{
  int i = glp_ios_best_node(tree);
  if (i)
    *(double*)info = glp_ios_node_bound(tree, i);
}
#endif

void Lag_Relax::solveSubProblem (bool &foundSolution, bool &foundOptSolution, double &objValue, double &bound)
{
  int ret;
#if 0
  if (m_solver == LpSolve || m_solver == Glpk)
  {
    ret = solve(m_lp);

    if (ret != OPTIMAL && ret != SUBOPTIMAL && ret != FEASFOUND)
    {
      foundSolution = false;
      foundOptSolution = false;
    }
    else
    {
      foundSolution = true;
      foundOptSolution = (ret == OPTIMAL) ? true : false;
      objValue = get_objective(m_lp);
      bound = (ret == OPTIMAL) ? objValue : m_lp->bb_limitOF;
      get_ptr_variables(m_lp, (double**)&m_curr_solution);
      
      // if required, try to improve the bound by using GLPK
      if (m_solver == Glpk)
      {
	write_freemps(m_lp, "mps.tmp");

	glp_prob *P;
	glp_iocp parm;
	double glpk_bound;

	P = glp_create_prob();
	glp_read_mps(P, GLP_MPS_FILE, NULL, "mps.tmp");

	glp_init_iocp(&parm);
	parm.tm_lim = 120 * 1000;
	parm.presolve = GLP_ON;
	//parm.gmi_cuts = GLP_ON;

	parm.cb_func = my_glpk_callback;
	parm.cb_info = (void*)&glpk_bound;

	// solve with branch and cut
	glp_intopt(P, &parm);

	glp_delete_prob(P);

	cout << endl << "Bound provided by branch and cut on the Lagrangian subproblem: " << glpk_bound << endl;

	if ((is_maxim(m_lp) && glpk_bound < bound) || (!is_maxim(m_lp) && glpk_bound > bound))
	  bound = glpk_bound;
      }
    }
  }
  else   // Coin-or Cbc
  {
#endif
//     write_freemps(m_lp, "mps.tmp");
// 
//     OsiClpSolverInterface solver1;
//     solver1.readMps("mps.tmp","");
// 
//     const char * argv2[]={"driver3","-solve","-quit"};
// 
//     CbcModel model(solver1);
// 
//     //model.setMaximumSeconds (600);
// 
//     CbcMain (3, argv2, model);
// 
//     // solve the model
//     OsiSolverInterface * solver = model.solver();

    OsiClpSolverInterface* solver = new OsiClpSolverInterface(m_model, true);
    
    CbcModel* cbcm = new CbcModel(*solver);
    
    callCbc("-solve", *cbcm);
    
    if (cbcm->bestSolution())
    {
      foundSolution = true;
      foundOptSolution = (!cbcm->status() && solver->isProvenOptimal()) ? true : false;
      objValue = solver->getObjValue();
      bound = cbcm->getBestPossibleObjValue();
      m_curr_solution = solver->getColSolution();
    }
    else
    {
      foundSolution = false;
      foundOptSolution = false;
    }
//   }
}

void Lag_Relax::solveLAG ()
{
  bool feasible;
  double sum;
  bool foundSolution, foundOptSolution;
  double subObjValue, subBound, fixedTerm;

  int fd = fileno(stdin);
  int fcflags = fcntl(fd,F_GETFL);
  //  set non-blocking input
  fcntl(fd,F_SETFL,fcflags | O_NONBLOCK);

  for (m_curr_iter=0; m_curr_iter<m_max_iter; m_curr_iter++)
  {
    cout << endl << "*** Iteration " << m_curr_iter+1 << " (press 'c' and 'Enter' to stop after this iteration) ***" << endl << endl;
    
    /** update the objective function with the current mu values **/
    update_obj_func();
  
    /** solve the Lagrangian subproblem **/
    solveSubProblem(foundSolution, foundOptSolution, subObjValue, subBound);

    if (!foundSolution)
    {
      if (!m_curr_iter)
      {
	cerr << endl << "ERROR: Solving the Lagrangian relaxation failed: Try with a different initial value for the multipliers" << endl << endl;
	return;
      }

      cout << endl << "* Cannot find a solution to the lagrangian subproblem. The current bound is " << m_lag_bound << " *" << endl << endl;

      // update the multipliers and proceed to the next iteration
      update_multipliers(true);
      continue;
    }

    /** compute the bound **/

    fixedTerm = 0.;
    for (vector<lag_constraint>::iterator r=m_relax_constr.begin(); r!=m_relax_constr.end(); r++)
//       fixedTerm -= r->mu * r->rhs;
      fixedTerm -= r->mu * (r->lower == -COIN_DBL_MAX ? r->upper : r->lower);

    subObjValue += fixedTerm;
    subBound += fixedTerm;
    
    cout << endl << (foundOptSolution ? "Optimal" : "Sub-Optimal") << " solution to the Lagrangian subproblem: " << subObjValue << endl;

    if (!foundOptSolution)
      cout << "Bound to the optimal solution of the Lagrangian subproblem: " << subBound << endl << endl;

    // m_curr_lag = subBound;
    m_curr_lag = subObjValue;      // use the value of the (sub-)optimal solution to the lagrangian subproblem to update the multipliers

//     if (!m_have_bound || (is_maxim(m_lp) && subBound < m_lag_bound) || (!is_maxim(m_lp) && subBound > m_lag_bound) )
    if (!m_have_bound || (m_model->optimizationDirection() == -1 && subBound < m_lag_bound) || (m_model->optimizationDirection() == 1 && subBound > m_lag_bound) )
    {
      m_lag_bound = subBound;
//       cout << "which is the new " << (is_maxim(m_lp) ? "upper" : "lower") << " bound for the original problem" << endl << endl;
      cout << "which is the new " << (m_model->optimizationDirection() == -1 ? "upper" : "lower") << " bound for the original problem" << endl << endl;
      m_noimprove = 0;
    }
    else
    {
//       cout << "The " << (is_maxim(m_lp) ? "upper" : "lower") << " bound for the original problem is still " << m_lag_bound << endl << endl;
      cout << "The " << (m_model->optimizationDirection() == -1 ? "upper" : "lower") << " bound for the original problem is still " << m_lag_bound << endl << endl;
      m_noimprove++;
    }

    if (!m_have_bound)
      m_have_bound = true;

    /** a solution has been found, check to see if it is a fesible solution for the original problem **/

    // for this purpose, check whether the relaxed constraints are verified
    feasible = true;
    for (vector<lag_constraint>::iterator r=m_relax_constr.begin(); r!=m_relax_constr.end(); r++)
    {
      sum = 0.;
      for (unsigned int i=0; i<r->row.size(); i++)
	sum += r->row[i] * m_curr_solution[r->colno[i]-1];

//       if ( (r->type==LE && sum > r->rhs) || (r->type==EQ && sum != r->rhs) || (r->type==GE && sum < r->rhs) )
      if (!(sum >= r->lower && sum <= r->upper))
      {
	feasible = false;
	cout << endl << "The solution of the Lagrangian subproblem is NOT feasible for the original problem" << endl << endl;
	break;
      }
    }
    if (feasible)
    {
      cout << endl << "The solution of the Lagrangian subproblem is feasible for the original problem" << endl;
      // compute the objective function of this solution
      sum = inner_product(m_obj_func.begin()+1, m_obj_func.end(), m_curr_solution, 0.);
      
      cout << "The value of this solution is : " << sum << endl;
      
      // if a feasible solution had not been found or this one is better than the previous one
//       if (!m_have_solution || (is_maxim(m_lp) && sum > m_obj_val) || (!is_maxim(m_lp) && sum < m_obj_val) )
      if (!m_have_solution || (m_model->optimizationDirection() == -1 && sum > m_obj_val) || (m_model->optimizationDirection() == 1 && sum < m_obj_val) )
      {
	// save this solution as the best one found so far
	m_obj_val = sum;
// 	copy(m_curr_solution, m_curr_solution+get_Ncolumns(m_lp), m_solution);
	copy(m_curr_solution, m_curr_solution + m_model->getNumCols(), m_solution);
	cout << "and it is the best solution found so far" << endl << endl;
      }
      else
	cout << "The value of the best solution found so far is still " << m_obj_val << endl << endl;

      m_have_solution = true;

      // if a breakValue has been defined, check if this solution is better
//       if (m_hasBreakValue && ((is_maxim(m_lp) && m_obj_val > m_breakValue) || (!is_maxim(m_lp) && m_obj_val < m_breakValue)))
      if (m_hasBreakValue && ((m_model->optimizationDirection() == -1 && m_obj_val > m_breakValue) || (m_model->optimizationDirection() == 1 && m_obj_val < m_breakValue)))
      {
	cout << endl << "A better solution than the required one (" << m_breakValue << ") has been found, hence we stop searching" << endl << endl;
	return;
      }
    }

    /** update the lagrangian multipliers **/
    if (m_noimprove < m_max_noimprove)
      update_multipliers();
    else
    {
      cout << endl << "* " << m_max_noimprove << " iterations without improvements in the lagrangian bound *" << endl << endl;
      update_multipliers(false, true);
      m_noimprove = 0;
    }
    
    if (getchar() == 'c')
    {
      cout << endl << "Stop searching as requested by the user" << endl << endl;
      return;
    }

  }
  //  restore blocking input
  fcntl(fd,F_SETFL,fcflags);
  
}

Lag_Relax::~Lag_Relax ()
{
  delete m_solution;
}


double newton::theta (bool repair)
{
  if (repair)
    m_lambda = m_lambda / 2;

  // compute the norm of Ax-b
  double norm2 = 0.;
  
  for (vector<double>::const_iterator v=m_Ax_b().begin(); v!=m_Ax_b().end(); v++)
    norm2 += pow (*v, 2.);

  if (m_have_solution() && m_obj_val() < m_ub)
    m_ub = m_obj_val();

  return m_lambda * (m_ub - m_curr_lag()) / norm2;
}
