#include "subgraph.h"

SubGraph::SubGraph(Topology* T, bool reversed)
: Graph(".WMemptygraph"),
  m_topology (T),
  m_reversed (reversed)
{
  clear();

  m_nVertexNum = T->GetNumNodes();
  
  for (int u=0; u<T->GetNumNodes(); u++)
    for (TGraphNeighborIterator v=T->Node(u)->NeighborBegin(); v!=T->Node(u)->NeighborEnd(); v++) {
      // construct the vertices
      BaseVertex* start_vertex_pt = reversed ? get_vertex(v->Id()) : get_vertex(u);
      BaseVertex* end_vertex_pt = reversed ? get_vertex(u) : get_vertex(v->Id());

      // add the edge weight (1 for now)
      m_mpEdgeCodeWeight[get_edge_code(start_vertex_pt, end_vertex_pt)] = 1;
      
      // update the fan-in or fan-out variables
      // Fan-in
      get_vertex_set_pt(end_vertex_pt, m_mpFaninVertices)->insert(start_vertex_pt);

      // Fan-out
      get_vertex_set_pt(start_vertex_pt, m_mpFanoutVertices)->insert(end_vertex_pt);
  }
    
  m_nVertexNum = m_vtVertices.size();
  m_nEdgeNum = m_mpEdgeCodeWeight.size();
}


SubGraph::SubGraph(const SubGraph& graph) : Graph(graph)
{
  for (BaseVertexPt2SetMapIterator i=m_mpFaninVertices.begin(); i!=m_mpFaninVertices.end(); i++)
    i->second = new set<BaseVertex*>(*(i->second));
  
  for (BaseVertexPt2SetMapIterator i=m_mpFanoutVertices.begin(); i!=m_mpFanoutVertices.end(); i++)
    i->second = new set<BaseVertex*>(*(i->second));

  m_stRemovedEdge.clear();
  m_stRemovedVertexIds.clear();
}


SubGraph::~SubGraph()
{
  m_mpFanoutVertices.clear();
  m_mpFaninVertices.clear();
  m_vtVertices.clear();
  m_mpVertexIndex.clear();
}


void SubGraph::removeEdge(int u, int v) {
  m_mpEdgeCodeWeight.erase(get_edge_code(get_vertex(u),get_vertex(v)));
  get_vertex_set_pt(get_vertex(v), m_mpFaninVertices)->erase(get_vertex(u));
  get_vertex_set_pt(get_vertex(u), m_mpFanoutVertices)->erase(get_vertex(v));
  
  m_nEdgeNum = m_mpEdgeCodeWeight.size();
}


void SubGraph::removeVertex(int u)
{
  // remove incoming edges
  BaseVertexPt2SetMapIterator pos = m_mpFaninVertices.find(get_vertex(u));
  
  if (pos != m_mpFaninVertices.end()) {
    for (set<BaseVertex*>::iterator v=pos->second->begin(); v!=pos->second->end(); v++)
      m_mpEdgeCodeWeight.erase(get_edge_code(*v,get_vertex(u)));
    pos->second->clear();
  }
  
  // remove outgoing edges
  pos = m_mpFanoutVertices.find(get_vertex(u));
  
  if (pos != m_mpFanoutVertices.end()) {
    for (set<BaseVertex*>::iterator v=pos->second->begin(); v!=pos->second->end(); v++)
      m_mpEdgeCodeWeight.erase(get_edge_code(get_vertex(u),*v));
    pos->second->clear();
  }

  m_nEdgeNum = m_mpEdgeCodeWeight.size();
}

void SubGraph::pruneEdges (linkPruneFunc* prune)
{
  if (!prune)
    return;

  for (TGraphNodeIterator u = m_topology->NodeBegin(); u != m_topology->NodeEnd(); u++)
    for (TGraphNeighborIterator v = u->NeighborBegin(); v != u->NeighborEnd(); v++)
      if ((*prune)(m_topology->LinkItrs(u,v)))
      {
	int start = m_reversed ? v->Id() : u->Id();
	int end = m_reversed ? u->Id() : v->Id();
	removeEdge (start, end);
      }
}

void SubGraph::setLinkCosts (linkCostFunc* cost)
{
  if (!cost)
    return;

  for (TGraphNodeIterator u = m_topology->NodeBegin(); u != m_topology->NodeEnd(); u++)
    for (TGraphNeighborIterator v = u->NeighborBegin(); v != u->NeighborEnd(); v++)
    {
      int start = m_reversed ? v->Id() : u->Id();
      int end = m_reversed ? u->Id() : v->Id();
      m_mpEdgeCodeWeight[get_edge_code(get_vertex(start), get_vertex(end))] = (*cost)(m_topology->LinkItrs(u,v));
    }
}
