#include "query-olsrd.h"

#include <climits>
#include <algorithm>
#include "ns3/ipv4-l3-protocol.h"

using namespace ns3;


/*questa funzione controlla se il nodo è presente nella topologia (facendo un controllo sull'ip)
se è presente restituisce l'indice del nodo, se non è presente restituisce il valore max che può assumere 
uno short int unsigned
*/
uint16_t top::trova(const Ipv4Address a){
	bool c=false;
	uint16_t o=USHRT_MAX;
	for (uint16_t i=0; (i<t.size()) && (!c);i++){		//scorre la topologia
		if(t[i].n.getIp()==a){		//se trova corrispondenza di ip memorizza l'indice dell'elemento
			c=true;
			o=i;
		}
	}
	return o;
}

/*dato un indirizzo ip questa funzione crea una nuova struct con tale indirizzo e con l'indice dell'elemento
 e la inserisce nella topologia 
 */
void top::crea(const Ipv4Address a){
	N sn;
	sn.n.setIp(a);			//inserisce l'indirizzo ip
	sn.n.setId(t.size()-1);		//inserisce l'indice del nuovo nodo
	t.push_back(sn);		//inserisce nella topologia
}


//Presi gli ip di due nodi vicini, questa funzione si occupa di inserire queste
//info nella topologia
void top::inserisci(const Ipv4Address a,const Ipv4Address b){
	uint16_t indice,id;
	id=trova(a);			//cerca l'indice di a	
	if(id==USHRT_MAX){		//se a non è presente
		crea(a);		//crea il nodo a
		id=t.size()-1;		//memorizza l'indice di a
		indice=trova(b);	//controlla se esiste il nodo b memorizzandone l'indice
		if(indice==USHRT_MAX){			//se il nodo non esiste
			crea(b);			//crea il nodo b
			indice=t.size()-1;		//memorizza l'indice di b
			t[id].v.insert(indice);		//inserisce b tra i vicini di a
			t[indice].v.insert(id);		//inserisce a tra i vicini di b
		}
		else{			//se il vicino b esiste già
			t[id].v.insert(indice);		//inserisce b tra i vicini di a
			t[indice].v.insert(id);		//inserisce a tra i vicini di b
		}
	}
	else{			//se il nodo a esiste già
		indice=trova(b);		//controlla se il nodo vicino b esiste e ne memorizza l'indice
		if(indice==USHRT_MAX){		//se b non esiste 
			crea(b);		//crea il nodo b
			indice=t.size()-1;	//memorizza l'indice di b
			t[id].v.insert(indice);	//mette b tra i vicini di a
			t[indice].v.insert(id); //inserisce a tra i vicni di b
		}
		else{	//se il vicino b esiste
			t[id].v.insert(indice);		//inserisce b tra i vicini di a
			t[indice].v.insert(id);		//inserisce a tra i vicini di b
		}
	}
}

//popola la topologia a partire dalle info di un singolo nodo contenute nel LinkSet, 2hopNs, e TopologySet
void top::popola(const olsr::OlsrState *s){
//	NS_LOG_UNCOND("popola");
	olsr::LinkSet ls=s->GetLinks();
	olsr::TwoHopNeighborSet tns=s->GetTwoHopNeighbors();	
	olsr::TopologySet ts=s->GetTopologySet();

	//scorre il link set
	for(olsr::LinkSet::const_iterator o= ls.begin();o<ls.end();o++){
		inserisci(o->localIfaceAddr,o->neighborIfaceAddr);
                //NS_LOG_UNCOND("LinkSet	"<<o->localIfaceAddr<<"	"<<o->neighborIfaceAddr);
	}
	//scorre il twohopneighbor set
	for(olsr::TwoHopNeighborSet::const_iterator p= tns.begin(); p<tns.end();p++){
		//NS_LOG_UNCOND("TwoHopNSet	"<<p->neighborMainAddr<<"	"<<p->twoHopNeighborAddr);
		inserisci(p->neighborMainAddr,p->twoHopNeighborAddr);
	}
	//scorre il topology set
	for(olsr::TopologySet::const_iterator q= ts.begin(); q<ts.end(); q++){
		//NS_LOG_UNCOND("TopologySet	"<<q->destAddr<<"	"<<q->lastAddr);
		inserisci(q->destAddr,q->lastAddr);
	}
}

//funzione di stampa
void top::stampa()const{
//NS_LOG_UNCOND("stampa");
	uint16_t p;
	set<uint16_t>::iterator q;
	for(vector<N>::const_iterator i=t.begin(); i<t.end();i++){
		NS_LOG_UNCOND("Nodo"<<i->n.getIp());
		p=i->v.size();
		q=i->v.begin();
		for(uint16_t a=0;a<p;a++){
			NS_LOG_UNCOND("vicino"<<*q);
			q++;
			}
	}
}


/*
   Applies Dijkstra's algorithm to the topology
   Returns a vector containing the distance and the predecessor of each node
   Links for which prune is true are not considered
   The link cost function is computed by the cost function object
*/
vector<TNodePro> top::Dijkstra(const uint16_t source)
{
	uint16_t current;

	/* initialize the vector of node properties */
	/* the i-th element of state is related to node i */
	vector<TNodePro> state(t.size());
	state[source].distance = 0.;

	/* create the queue containing node IDs */
	set<uint16_t> Q;
	set<uint16_t>::iterator p;
	for (uint16_t i=0; i<t.size(); i++)
		Q.insert(i);

	while (!Q.empty()){
		current = *(min_element(Q.begin(),Q.end(),lessDistant(&state)));
		p=t[current].v.begin();
		for (uint16_t j=0; j<=t[current].v.size(); j++){
			if ( state[*p].distance > state[current].distance + 1){
				state[*p].distance = state[current].distance + 1;
				state[*p].prev_link= current;
			}
			p++;
		}
		Q.erase(current);
	}
	return state;
}

void top::stampaDistanze(int id){
	vector<TNodePro> v=Dijkstra(id);
	int n=0;
	std::cout<<"Nodo " << id <<"\n"; 
	for(vector<TNodePro>::const_iterator i= v.begin(); i<v.end();i++){
		std::cout<<"Nodo	"<<n<<"dista	"<<i->distance;
		std::cout<<"precedente	"<<i->prev_link<<std::endl;
		n++;
	}
}


Mac48Address getMac(const Ipv4Address ip, const NodeContainer nc ){
	Mac48Address mac;
	for(NodeContainer::Iterator i=nc.Begin();i<nc.End();i++){
		Ipv4Address ipv= ((*i)-> GetObject<Ipv4L3Protocol>()->GetAddress(1,0)).GetLocal();
		if(ip==ipv){
			mac= Mac48Address::ConvertFrom((*i)->GetDevice(1)->GetAddress());
		}
	}
	return mac;
}


void QueryOlsrd(NodeContainer nodes){
	Mac48Address mac;
	vector<TNodePro> ve; 
	Ptr<layer::LayerProtocol> layer;

	for (NodeContainer::Iterator it=nodes.Begin(); it!=nodes.End(); it++){
		int d=0;
		//indice elemento nel nodecontainer
		top topologia;
		Mac48Address mac2;
		// TODO GetState ??
		//OlsrState * olsr_state=(*it)->GetObject<olsr::RoutingProtocol>()->GetState();
		olsr::OlsrState * olsr_state;
		topologia.popola(olsr_state);
		//topologia.stampa();
		//topologia.stampaDistanze(it-nodes.Begin());
		//Mac48Address mac=getMac(topologia.t[1].n.getIp(),nodes);
		//std::cout<<"ECCOLO"<<mac<<endl;
		for(uint16_t i=0; i<topologia.t.size();i++){
			ve=topologia.Dijkstra(i);
			mac=getMac(topologia.t[i].n.getIp(),nodes);
			layer = (*it)->GetDevice(0)->GetObject<layer::LayerProtocol>();
			layer->AddDestination(mac,ve[0].distance);
		//	std::cout<<"mac	"<<mac<<endl;
			set<uint16_t>::iterator a=topologia.t[0].v.begin();
			for(uint16_t p = 0; p!=topologia.t[0].v.size();p++){
				if(ve[0].distance>ve[*a].distance)
					d=(-1);
					else
						if(ve[0].distance<ve[*a].distance)
							d=1;
				mac2=getMac(topologia.t[*a].n.getIp(),nodes);
		//		std::cout<<"mac2	"<<mac2<<endl;
				layer = (*it)->GetDevice(0)->GetObject<layer::LayerProtocol>();
				layer->AddNeighborRtEntry(mac,mac2,d);			
				a++;
			
			}
			ve.clear();
    		}
    		
	}
}
