#include "wimesh-rtproto-ns3.h"
#include "ns3/mesh-helper.h"
#include "ns3/mesh-wifi-interface-mac.h"
#include "WMtypes.h"
#include <iostream>

#ifdef HAVE_LAYER2_5
#include "ns3/olsr-helper.h"
#include "query-olsrd.h"
#endif


WiMeshRtProtoNs3::WiMeshRtProtoNs3 (string _stack)
  : WiMeshNs3 ()
#ifdef HAVE_LAYER2_5
  , useOlsr (false)
#endif
{
  // ignore invalid values
#ifdef HAVE_LAYER2_5
  if (_stack == "ns3::LayerStack" || _stack == "ns3::FlameStack" || _stack == "ns3::Dot11sStack")
#else
  if (_stack == "ns3::FlameStack" || _stack == "ns3::Dot11sStack")
#endif
    stack = _stack;  
  else
  {
    std::cerr << "Ignoring invalid stack type (" << _stack << "), using default ns3::Dot11sStack" << std::endl;
    stack = "ns3::Dot11sStack";
  }
}

void WiMeshRtProtoNs3::CreateNs3MeshDevices ()
{
  Ptr<WifiNetDevice> wnd;
  int ch;

  MeshHelper mesh_helper = MeshHelper::Default ();

  mesh_helper.SetMacType ();

  SetRemoteStationManager (mesh_helper);

//  mesh_helper.SetRemoteStationManager ("ns3::WiMeshWifiManager",
//                                 "WiMeshNs3Ptr", WiMeshNs3PtrValue (WiMeshNs3Ptr (this)));
//  mesh_helper.SetRemoteStationManager("ns3::ConstantRateWifiManager", "DataMode",  StringValue ("OfdmRate54Mbps") );

  mesh_helper.SetStackInstaller (stack);

  // create mesh devices
  for (TGraphNodeIterator u = topology->NodeBegin (); u != topology->NodeEnd (); u++)
  {
    mesh_helper.SetNumberOfInterfaces (u->Info()->numAvailRadios() - u->Info()->countRadiosWithChannel(0));
    // create the required number of devices for the mesh node (radios witch channel 0 are dealt with later)
    // the resulting meshPointDevice (a pseudo interface) is added to meshPointDevices
    NetDeviceContainer mp = mesh_helper.Install (wifiPhy, NodeContainer (meshnodes.Get (u->Id())));
    meshPointDevices.Add (mp);
    // assign channels to devices
    unsigned int r = 0;
    for (uint32_t i = 0; i < meshnodes.Get (u->Id())->GetNDevices (); ++i)
    {
      wnd = meshnodes.Get (u->Id())->GetDevice (i)->GetObject<WifiNetDevice> ();
      // if it is a WifiNetDevice
      if (wnd != 0)
      {
	// skip channel 0
	do {
	  ch = u->Info()->Radio(r++)->Channel();
	} while (!ch);

	wnd->GetPhy ()->SetChannelNumber (ch * channelsep);
	meshdevices.Add(wnd);
      }
    }

    if (createRadiosOff)
      // Add radios set to channel 0. Do not install the mesh stack as they are turned off
      for (int j = 0; j < u->Info()->countRadiosWithChannel(0); j++)
      {
	Ptr<WifiNetDevice> iface = CreateInterfaceWithoutStack (wifiPhy, meshnodes.Get (u->Id()), 0);
	Ptr<MeshPointDevice> mpd = DynamicCast<MeshPointDevice> (*mp.Begin());
	mpd->AddInterface (iface);
	meshdevices.Add(iface);

	// add the device to the set of radios to be put in the sleep mode
	radiosInitiallyOff.Add (iface);
      }

    // assign random variable stream number to devices
    for (uint32_t i = 0; i < meshnodes.Get (u->Id())->GetNDevices (); ++i)
    {
      wnd = meshnodes.Get (u->Id())->GetDevice (i)->GetObject<WifiNetDevice> ();
      // if it is a WifiNetDevice
      if (wnd != 0)
      {
	int64_t currentStream = GetRadioRandomVarStreamNumber (u->Id(), wnd->GetPhy ()->GetChannelNumber () / channelsep);

	// assign a fixed random variable stream number depending on the channel
	// (inspired by ns3::MeshHelper::AssignStreams)
	// Handle any random numbers in the PHY objects.
	currentStream += wnd->GetPhy ()->AssignStreams (currentStream);

	// Handle any random numbers in the station managers.
	Ptr<WifiRemoteStationManager> manager = wnd->GetRemoteStationManager ();
	Ptr<MinstrelWifiManager> minstrel = DynamicCast<MinstrelWifiManager> (manager);
	if (minstrel)
	{
	  currentStream += minstrel->AssignStreams (currentStream);
	}
	// Handle any random numbers in the mesh mac and plugins
	Ptr<MeshWifiInterfaceMac> mac = DynamicCast<MeshWifiInterfaceMac> (wnd->GetMac ());
	if (mac)
	{
	  currentStream += mac->AssignStreams (currentStream);
	}
      }
    }
  }

  //install IP stack and olsr routing protocol
  InternetStackHelper internetStack;
  #if 0
  if(stack=="ns3::LayerStack" && useOlsr){
  	OlsrHelper ols;
  	Ipv4StaticRoutingHelper staticRouting;
  	Ipv4ListRoutingHelper list;
  	list.Add (staticRouting, 0);
  	list.Add (ols, 10);
  	internetStack.SetRoutingHelper (list);
  }
  #endif

  internetStack.Install (meshnodes);
  Ipv4AddressHelper address;
  address.SetBase ("10.1.1.0", "255.255.255.0");
  Ipv4InterfaceContainer interfaces = address.Assign (meshPointDevices);
}

void WiMeshRtProtoNs3::ConfigureForwarding ()
{
#ifdef HAVE_LAYER2_5
  if (stack == "ns3::LayerStack")
  {
    ConfigureL2_5Links ();
    
    if (!useOlsr)
      ConfigureL2_5Topology ();
    else
      Simulator::Schedule(Seconds(30), &QueryOlsrd, meshnodes);
  }
#endif

  SetRoutes();
}

#ifdef HAVE_LAYER2_5
void WiMeshRtProtoNs3::EnableOlsr ()
{
  useOlsr = true;
}


void WiMeshRtProtoNs3::ConfigureL2_5Links ()
{
  // NOTE è uguale a ConfigureLinksWiMesh senza la parte sul RemoteManager
//   set<unsigned int> nodech;
//   set<unsigned int> neighborch;
  Ptr<layer::LayerProtocol> layer;

  // adding routing table entries
  for (TGraphNodeIterator i = topology->NodeBegin (); i != topology->NodeEnd (); i++)
  {
    // find the set of channels (<>0) this node uses
//     findChannels(i,nodech);
//     nodech = i->info->findChannels();

    // add links to nodes
    for (TGraphNeighborIterator j = i->NeighborBegin (); j != i->NeighborEnd (); j++)
    {
      //find the channels used by the neighbor
//       findChannels(j, neighborch);
//       neighborch = j->info->findChannels();
      //main address of neighbor
      layer::LayerAddress neigh=Mac48Address::ConvertFrom(meshnodes.Get(j->Id())->GetDevice(1)->GetAddress());

      //address of the neighbor's interface
//       layer::LayerAddress neigh_ifn=Mac48Address::ConvertFrom(meshnodes.Get(j->info->id)->GetDevice(findIFindex(neighborch, j->radio->channel)+1)->GetAddress());

//       layer::LayerAddress neigh_ifn=Mac48Address::ConvertFrom(meshnodes.Get(j->Id())->GetDevice( j->Info()->findRadioWithChannel(j->RadioChannel())+1)->GetAddress());

      uint32_t r;
      for (r = 0; r < meshnodes.Get (j->Id())->GetNDevices (); ++r)
      {
	Ptr<WifiNetDevice> wnd = meshnodes.Get(j->Id())->GetDevice (r)->GetObject<WifiNetDevice> ();
	// if it is a WifiNetDevice
	if (wnd != 0 && wnd->GetPhy ()->GetChannelNumber () / channelsep == j->RadioChannel())
	  break;
      }

      layer::LayerAddress neigh_ifn=Mac48Address::ConvertFrom(meshnodes.Get(j->Id())->GetDevice(r)->GetAddress());

      std::cout<<"Adding neighbor " << j->Id() << " with main address " << neigh << " and interface address " << neigh_ifn << " to node " << i->Id()  << " on radio channel " << j->RadioChannel() << " with flow " << j->Flow() << " rate " << j->Rate() << " and power " << WMS.TxPowerLevel(j->TxPower()) << "\n";      

      layer = meshnodes.Get (i->Id())->GetDevice (0)->GetObject<layer::LayerProtocol>();

      for (r = 0; r < meshnodes.Get (i->Id())->GetNDevices (); ++r)
      {
	Ptr<WifiNetDevice> wnd = meshnodes.Get(i->Id())->GetDevice (r)->GetObject<WifiNetDevice> ();
	// if it is a WifiNetDevice
	if (wnd != 0 && wnd->GetPhy ()->GetChannelNumber () / channelsep == j->RadioChannel())
	  break;
      }

      layer->AddNeighborLink (neigh, neigh_ifn, r-1, j->Flow(), j->Rate(), DbmToW(WMS.TxPowerLevel(j->TxPower())) );
    }
  }
}

void WiMeshRtProtoNs3::ConfigureL2_5Topology ()
{
  // NOTE è uguale a ConfigureTopologyWiMesh
  Ptr<layer::LayerProtocol> layer;
  
  /* computes distances using Dijkstra's */
  set<int> destinations;
  for (int i=0; i<topology->GetNumAggregationNodes(); i++)
    destinations.insert(topology->GetAggregationNode(i));
  for (int i=0; i<topology->GetNumGatewayNodes(); i++)
    destinations.insert(topology->GetGatewayNode(i));

  linkPruneFunc* prune = new linkPrune_Bwd(0.);

  // Add the destinations route entries
  for (set<int>::iterator dst=destinations.begin(); dst!=destinations.end(); dst++){

    //get address of destination
    layer::LayerAddress dest=Mac48Address::ConvertFrom(meshnodes.Get(*dst)->GetDevice(1)->GetAddress());
    vector<float> distance = topology->Dijkstra(*dst, true, prune);
    
    // add the destinations
    for (unsigned int i=0; i<topology->GetNumNodes(); i++)
      if ((int)i != *dst)
	{
	  layer = meshnodes.Get (i)->GetDevice(0)->GetObject<layer::LayerProtocol>();
	  layer->AddDestination(dest, (int)distance[i]);
	}

    // Add the neighbor entries (NeighborRtEntry-s) for each destination
    // for each destination the type of each neighbor is given (tuple: destination neighbor_main_address type) 
    for (TGraphNodeIterator i=topology->NodeBegin(); i!=topology->NodeEnd(); i++)
      if (i->Id() != *dst)
      {
	for (TGraphNeighborIterator j=i->NeighborBegin(); j!=i->NeighborEnd(); j++)
	{
	  // neighbor main address
	  layer::LayerAddress neighbor=Mac48Address::ConvertFrom(meshnodes.Get(j->Id())->GetDevice(1)->GetAddress());

	  int type=(int)distance[j->Id()] - (int)distance[i->Id()];

	  std::cout<<"Adding to node " << i->Id() << " for destination " << dest << " neighbor " << neighbor << " of type " << type<<".\n";

	  layer = meshnodes.Get(i->Id())->GetDevice(0)->GetObject<layer::LayerProtocol>();
	  layer->AddNeighborRtEntry(dest, neighbor, type);
	}
      }
  }

  delete prune;
}
#endif

void WiMeshRtProtoNs3::SetRoutes()
{
// add the routing rules required to reach the nodes attached to the gateways
  Ipv4StaticRoutingHelper ipv4RoutingHelper;
  Ptr<Ipv4StaticRouting> staticRouting;
  
  for (TDemands::iterator d=demands.begin(); d != demands.end(); d++){

	// get the aggregator and gateway associated to the "demand" 
	int dst,src;
	src=d->first.first;
	dst=d->first.second;
	
	// get the ip address of the mesh interface of the gateway
	Ptr<Ipv4> ipv4 = meshnodes.Get (dst)->GetObject<Ipv4> ();
	Ipv4Address gateway_addr = ipv4->GetAddress (1, 0).GetLocal ();
	
	// get the static routing object associated to the aggregator
	Ptr<Node> agg = meshnodes.Get(src);
	Ptr<Ipv4> ipv4Aggr=agg->GetObject<Ipv4>();	
	staticRouting = ipv4RoutingHelper.GetStaticRouting (ipv4Aggr);

	// create the strings with the addresses of the point-to-point link
	stringstream host, prefix;
	prefix<< "192.168."<<dst;
	host << prefix.str() << ".1";
	
	// add the routing rule
	std::cout<<"Adding a routing rule to reach " << host.str() << " through " << gateway_addr  << " on node " << src <<  "." << std::endl;
	staticRouting->AddHostRouteTo (Ipv4Address (host.str().c_str ()), gateway_addr, 1);
  }
}


void WiMeshRtProtoNs3::DoReceiveFinalDataFailed(int me, Mac48Address neighbor)
{
#ifdef HAVE_LAYER2_5
  if (stack=="ns3::LayerStack")
  {
    Ptr<layer::LayerProtocol> layer = meshnodes.Get(me)->GetDevice(0)->GetObject<layer::LayerProtocol>();
    layer->ChangeNeighborStatus(neighbor, false);
  }
#endif
}



void WiMeshRtProtoNs3::ChangeTopology(Topology* T, int* changes)
{
  /*
   * switch channels and update IP addresses
   */
  vector<multiset<unsigned int> > newChSet, oldChSet;

  int res = topology->countRadioChanged (T, oldChSet, newChSet);
  
  if (changes)
    *changes = res;
  
  for (int u = 0; u < (int)T->GetNumNodes(); u++)
  {
    // do the channel switches
    multiset<unsigned int>::iterator newCh = newChSet[u].begin();
    for (multiset<unsigned int>::iterator oldCh = oldChSet[u].begin(); oldCh != oldChSet[u].end(); oldCh++, newCh++)
      DoChannelSwitch (u, *oldCh, *newCh);      
  }

  // the topology pointed to by WiMeshNs3 is the new one.
  // Keeping updated the topology member is required because, after a channel switch,
  // the WiMeshWifiManager tries to find the link rate. If the WiMesh topology is not updated,
  // a TGraphNeighbor with the new channel is not found and the basic rate (6Mbps) is used
  topology = T;
}

Ptr<WifiNetDevice> WiMeshRtProtoNs3::CreateInterfaceWithoutStack (const WifiPhyHelper &phyHelper, Ptr<Node> node, uint16_t channel)
{
  Ptr<WifiNetDevice> device = CreateObject<WifiNetDevice> ();

  ObjectFactory m_mac;
  m_mac.SetTypeId ("ns3::MeshWifiInterfaceMac");
  Ptr<MeshWifiInterfaceMac> mac = m_mac.Create<MeshWifiInterfaceMac> ();
  NS_ASSERT (mac != 0);
  mac->SetSsid (Ssid ());
  
  ObjectFactory m_stationManager;
  m_stationManager.SetTypeId (RSMname);
  if (RSMname == "ns3::WiMeshWifiManager")
    // WiMeshWifiManager requires a pointer to a WiMeshNs3 object
    m_stationManager.Set ("WiMeshNs3Ptr", WiMeshNs3PtrValue (WiMeshNs3Ptr (this)));
  Ptr<WifiRemoteStationManager> manager = m_stationManager.Create<WifiRemoteStationManager> ();
  NS_ASSERT (manager != 0);
  // set the options specified in the XML config file
  for (vector<Ns3Attribute>::iterator i = RSMoptions.begin(); i != RSMoptions.end(); i++)
    manager->SetAttribute (i->m_name, *PeekPointer(i->m_ptr_value));

  Ptr<WifiPhy> phy = phyHelper.Create (node, device);
  mac->SetAddress (Mac48Address::Allocate ());
  mac->ConfigureStandard (WIFI_PHY_STANDARD_80211a);
  phy->ConfigureStandard (WIFI_PHY_STANDARD_80211a);
  device->SetMac (mac);
  device->SetPhy (phy);
  device->SetRemoteStationManager (manager);
  node->AddDevice (device);
  mac->SwitchFrequencyChannel (channel);
  return device;
}
